# Accounting.pm - The Object class that provides the Accounting Application Object.
# Created by James A. Pattie, 07/11/2000.
package Portal::Accounting;
#perl module to run the accounting application.
#$Id: Accounting.pm,v 1.39.2.1 2005/12/05 20:56:20 moreejt Exp $
#This package is released under the GPL.  You should have recieved the GPL with it.
# Copyright (c) 2000, 2001 HLR http://www.pcxperience.org  All rights reserved.

use strict;
use Portal::App;
use Portal::Accounting::DB::Main;
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);
use vars qw($release $DBVersion $codeVersion);

require Exporter;

@ISA = qw(Portal::App Exporter AutoLoader);
@EXPORT = qw();

##VERSION INFO##
$release = "1";
$DBVersion = "5";
$codeVersion = "1";
$VERSION = '1.5.1';

# global variables
my %states = ( 
               CGL => "Capital Gain/Loss",
               "Config::Monitor" => "Config Monitor Menu",
               Config => "Configuration Subsystem",
               DisplayItems => "Display Items",
               Ledger => "Ledger",
               Main => "Main Menu",
               InvJournal => "Investment Journal",
               'Investments::Journal' => "Investment Journal",
               'Investments::WatchList' => "Watchlists",
               JournalEntry => "Journal Entry",
               Portfolio => "Portfolio",
               QIFImport => "QIF Import",
               DBUpgrade => "DBUpgrade",
               StockPrices => "Stock Prices",
               );

=head1 NAME

Accounting - Object used to build a Accounting Class.

=head1 SYNOPSIS

  use Portal::Accounting;
  my $obj = Portal::Accounting->new;
  if ($obj->didErrorOccur())
  {
    die $obj->errorMessage();
  }

=head1 DESCRIPTION

Accounting is the main class for the Accounting Application.  All
work in the Accounting Application is funneled through this module.
This module takes care of setting up the database connection, session
info, etc. and then calls the module that will do the actual work for
the user.

=head1 Exported FUNCTIONS

B<NOTE>: I<bool> = 1(true), 0(false)

=over 4

=item scalar new()

 Creates a new instance of the Portal::Accounting object.
 See Portal::App(3) for a listing of required arguments.

=cut

sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  my %args = ( @_ );

  if ($self->didErrorOccur)
  {
    $self->prefixError();
    return $self;
  }

  # instantiate anything unique to this module
  $self->{appName} = "Accounting";
  $self->{appVersion} = $VERSION;
  $self->{states} = \%states;
  $self->{logOk} = 0;  # we can not log safely.  Change to 1 if you do have a log_tb defined in your database.
  $self->{useAppStateCommandSecurity} = 0;
  $self->{setDateStyle} = ($self->{input}->{state} =~ /^(CGL)$/ ? 0 : 1);

  $self->{preStateCallback} = "checkDatabaseVersion";
  $self->{stateEvalErrorCallback} = "unlockApplication";

  $self->{extraArgs} = "release => '$release', DBVersion => '$DBVersion', codeVersion => '$codeVersion'";

  # do validation
  if (!$self->Portal::Accounting::isValid)
  {
    # the error is set in the isValid() method.
    return $self;
  }

  # do anything else you might need to do.

  $self->instantiatePortalApp(dbHost => "App");
  if ($self->didErrorOccur)
  {
    $self->prefixError();
    return $self;
  }

  return $self;
}

=item % getStates(void)

  This method returns the states hash and is only to be
  used by the registerApp method for creating the permissions,
  if using the AppStateCommandSecurity implementation.

=cut
sub getStates
{
  return %states;
}

=item bool isValid(void)

 Returns 0 or 1 to indicate if the object is valid.
 The error will be available via errorMessage().

=cut

sub isValid
{
  my $self = shift;

  # make sure our Parent class is valid.
  if (!$self->SUPER::isValid())
  {
    $self->prefixError();
    return 0;
  }

  # validate our parameters.

  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->postfixError($self->genErrorString("all"));
    return 0;
  }

  return 1;
}

=item (state, command) checkDatabaseVersion(state, command)

 Do any processing to make sure the database is upgraded.
 If we do need to upgrade the database, lock the application
 and point the user to the DB Upgrade code.  Remember where
 they were trying to go.

 returns: array containing the state, command to goto.

=cut
sub checkDatabaseVersion
{
  my $self = shift;
  my %args = ( state => "", command => "", @_ );
  my $state = $args{state};
  my $command = $args{command};
  my @result = ($state, $command);
  my $currVersion  = $release . "." . $DBVersion;

  # check and see if we need to upgrade the database.
  my $mainDBObj = Portal::Accounting::DB::Main->new(dbHandle => $self->{dbHandle}, langObj => $self->{langObj});
  if ($mainDBObj->didErrorOccur)
  {
    $self->setError($mainDBObj->errorMessage);
    return @result;
  }

  # check Database version
  my $dbVersion = $mainDBObj->getDBVersion;
  if ($mainDBObj->didErrorOccur)
  {
    $self->setError($mainDBObj->errorMessage);
    return @result;
  }
  if ($dbVersion < $currVersion)
  {
    if ($self->{userObj}->{admin})
    {
      # see if we are already in the DBUpgrade process.
      my @lockedAppInfo = $self->{applicationObj}->getLockedAppInfo(app => $self->{appObj}->{id});
      if ($self->{applicationObj}->didErrorOccur)
      {
        $self->setError($self->{applicationObj}->errorMessage);
        return @result;
      }
      my $appLocked = 0;
      foreach my $lockedApp (@lockedAppInfo)
      {
        if ($lockedApp->{type} eq "c" && $lockedApp->{company} == $self->{companyObj}->{id} && $lockedApp->{admin} == $self->{userObj}->{id})
        {
          $appLocked = 1;
        }
      }
      if (!$appLocked)
      {
        # lock XIWA for the current company
        $self->{applicationObj}->lockApp(app => $self->{appObj}->{id}, type => 'c',
                                         company => $self->{companyObj}->{id});
        if ($self->{applicationObj}->didErrorOccur)
        {
          $self->setError($self->{applicationObj}->errorMessage);
          return @result;
        }

        # store the users state,command,arguments for calling later in their session.
        my $originalDestination = "";
        foreach my $arg (keys %{$self->{input}})
        {
          next if ($arg eq "_FileUploadInfo_");
          $originalDestination .= "&" if (length $originalDestination > 0);
          $originalDestination .= "$arg=" . $self->{input}->{$arg};
        }

        $self->{appSessions}->{apps}->{$self->{appName}}->{store}->{originalDestination} = $originalDestination;

        # signal we need to dump the database first.
        $self->{appSessions}->{apps}->{$self->{appName}}->{store}->{dumpDB} = 1;

        # push to the DBUpgrade module.
        $state = "Portal::$self->{appName}::DBUpgrade";
        $command = "display";
      }
    }
    else
    {
      # display an error about how the database needs upgrading but they are not an administrator.
      # store the users state,command,arguments for calling later in their session.
      my $originalDestination = "";
      foreach my $arg (keys %{$self->{input}})
      {
        next if ($arg eq "_FileUploadInfo_");
        $originalDestination .= "&" if (length $originalDestination > 0);
        $originalDestination .= "$arg=" . $self->{input}->{$arg};
      }

      $self->{appSessions}->{apps}->{$self->{appName}}->{store}->{originalDestination} = $originalDestination;

      # push to the Upgrade needed screen..
      $state = "Portal::$self->{appName}::DBUpgrade";
      $command = "notAdmin";
    }
  }

  # return the state/command sent in so that we still go where we were trying to.
  return ($state, $command);
}

=item void unlockApplication(state, command)

 unlocks the app if we were locked since an
 error occured.

=cut
sub unlockApplication
{
  my $self = shift;
  my %args = ( state => "", command => "", @_ );
  my $state = $args{state};
  my $command = $args{command};

  # see if the database is locked, if so, unlock it.
  if ($state eq "Portal::Accounting::DBUpgrade")
  {
    # now unlock the app.
    $self->{applicationObj}->unlockApp(app => $self->{appObj}->{id}, type => "c", company => $self->{companyObj}->{id});
    if ($self->{applicationObj}->didErrorOccur)
    {
      $self->postfixError($self->{applicationObj}->errorMessage);
      return;
    }
  }
}

=back

=cut

1;
__END__

=head1 NOTE

 All data fields are accessible by specifying the object
 and pointing to the data member to be modified on the
 left-hand side of the assignment.
 Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

Xperience, Inc. (mailto:admin@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal(3), Portal::Accounting(3), Portal::App(3)

=cut
