# Main.pm - The Object Class that provides a Main Object
#$Id: Base.pm,v 1.12 2004/02/04 00:25:03 james Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.
# Created by James Pattie & Jason A. Pattie, 11/07/2000.

package Portal::Accounting::Base;
use strict;
use HTMLObject::Base;
use HTMLObject::Normal;
use HTMLObject::FrameSet;
use Portal::AppState;
use Portal::Data::Variables;
use Portal::Accounting::Data::Variables;
use Portal::Accounting::Objects::Menu;
use Date::Calc qw(:all);
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Portal::AppState Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '$Revision: 1.12 $';

# global variables.

=head1 NAME

Main - Object used to build a Main Menu for accounting package

=head1 SYNOPSIS

  use Portal::Accounting::Main;
  my $obj = Portal::Accounting::Main->new;
  if ($obj->didErrorOccur())
  {
    die $obj->errorMessage();
  }
  $obj->run(command => $command);

=head1 DESCRIPTION

Main is the Main Menu from which every other part of
the systems will be accessed.

=head1 Exported FUNCTIONS

B<NOTE>: I<bool> = 1(true), 0(false)

=over 4

=item scalar new(codeVersion, release, DBVersion)

 Creates a new instance of the Portal::Accounting::Main module.
 See Portal::AppState(3) for a listing of required arguments.

 Instantiates an instance of the Main State. !!Anything passed in here should be
 data that never changes from one run of a command to another such as database handles
 portalSession? input? cookieObj? dont fit that criterion

=cut

sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  my %args = ( release => "", DBVersion => "", @_ );

  if ($self->didErrorOccur)
  {
    $self->prefixError();
    return $self;
  }

  # instantiate anything unique to this module
  $self->{release} = $args{release};
  $self->{DBVersion} = $args{DBVersion};
  $self->{codeVersion} = $args{codeVersion};
  $self->{portalVariables} = Portal::Data::Variables->new(langObj => $self->{langObj});
  if ($self->{portalVariables}->didErrorOccur)
  {
    $self->setError($self->{portalVariables}->errorMessage);
    return $self;
  }
  $self->{variables} = Portal::Accounting::Data::Variables->new(langObj => $self->{langObj});
  if ($self->{variables}->didErrorOccur)
  {
    $self->prefixError($self->{variables}->errorMessage);
    return $self;
  }
  $self->{jsDir}         = $self->{methods}->createBaseURL(type => "App",    linkType => "js",       appConfigObj => $self->{sessionObj}->{store}->{companyAppObj} );
  $self->{imageDir}      = $self->{methods}->createBaseURL(type => "App",    linkType => "image",    appConfigObj => $self->{sessionObj}->{store}->{companyAppObj} );
  $self->{cssDir}        = $self->{methods}->createBaseURL(type => "App",    linkType => "css",      appConfigObj => $self->{sessionObj}->{store}->{companyAppObj} );
  $self->{dbDump}        = $self->{methods}->createBaseURL(type => "App",    linkType => "db_dump",  appConfigObj => $self->{sessionObj}->{store}->{companyAppObj} );
  $self->{oldCGIDir}     = $self->{methods}->createBaseURL(type => "App",    linkType => "js",       appConfigObj => $self->{sessionObj}->{store}->{companyAppObj} )."accounting/cgi-bin";
  $self->{portalCGIDir}  = $self->{methods}->createBaseURL(type => "Portal", linkType => "cgi",      appConfigObj => $self->{sessionObj}->{store}->{companyAppObj} );
  $self->{baseURL}       = $self->{methods}->createBaseURL(type => "App",    linkType => "cgi",      appConfigObj => $self->{sessionObj}->{store}->{companyAppObj});

  ## - moved from JT's run() method by James since we want to use the Portal::AppState::run() method instead.
  #the state is not set until runtime
  $self->{postInfo} .= "<input type=\"hidden\" name=\"app\" value=\"$self->{appName}\">\n";
  $self->{postInfo} .= "<input type=\"hidden\" name=\"state\" value=\"$self->{state}\">\n";
  $self->{getInfo} = "app=$self->{app}&state=" . HTMLObject::Base::encodeString($self->{state}) . $self->{getInfo};

  # do validation
  if (!$self->Portal::Accounting::Base::isValid)
  {
    # the error is set in the isValid() method.
    return $self;
  }

  return $self;
}

=item bool isValid(void)

 Returns 0 or 1 to indicate if the object is valid.
 The error will be available via errorMessage().

=cut

sub isValid
{
  my $self = shift;

  # make sure our Parent class is valid.
  if (!$self->SUPER::isValid())
  {
    $self->prefixError();
    return 0;
  }

  # validate our parameters.
  if (length $self->{release} == 0)
  {
    $self->invalid("release", $self->{release});
  }
  if (length $self->{DBVersion} == 0)
  {
    $self->invalid("DBVersion", $self->{DBVersion});
  }
  if (length $self->{codeVersion} == 0)
  {
    $self->invalid("codeVersion", $self->{codeVersion});
  }
  if (not defined $self->{methods})
  { $self->missing("methods");  }
  if (not defined $self->{oldCGIDir})
  { $self->missing("oldCgiDir");  }
  if (not defined $self->{portalCGIDir})
  { $self->missing("portalCgiDir");  }
  if (not defined $self->{baseURL})
  { $self->missing("baseURL");  }
  if (not defined $self->{jsDir})
  { $self->missing("jsDir");  }
  if (not defined $self->{imageDir})
  { $self->missing("imageDir");  }
  if (not defined $self->{cssDir})
  { $self->missing("cssDir");  }
  if (not defined $self->{dbDump})
  { $self->missing("dbDump");  }
  if (not defined $self->{sessionObj})
  { $self->missing("sessionObj");  }
  if (not defined $self->{sessionObj}->{store})
  { $self->missing("store");  }
  if (not defined $self->{sessionObj}->{store}->{companyAppObj})
  { $self->missing("companyAppObj");  }

  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->postfixError($self->genErrorString("all"));
    return 0;
  }

  return 1;
}

=back

=cut

1;
__END__

=head1 NOTE

 All data fields are accessible by specifying the object
 and pointing to the data member to be modified on the
 left-hand side of the assignment.
 Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

Xperience, Inc. (mailto:xiwa@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal(3), Portal::Accounting(3), Portal::AppState(3)

=cut
