# Main.pm - The Object Class that provides a Main Object
# Created by James Pattie & Jason A. Pattie, 11/07/2000.
#$Id: Main.pm,v 1.35.2.2 2004/12/22 21:25:58 moreejt Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.

package Portal::Accounting::DB::Main;
use strict;
use Portal::Base;
use Portal::Methods;
use Portal::Accounting::Objects::Ledger;
use Portal::Accounting::Data::Config;
use Portal::Data::Variables;
use Portal::Accounting::Objects::AccountObject;
use Portal::Accounting::Objects::SubAccountObject;
use Portal::Accounting::Objects::SubEntityObject;
use Portal::Accounting::Objects::FiscalYearObject;
use Portal::Accounting::Objects::StockObject;
use Portal::Accounting::Objects::MonitorObject;
use Date::Calc qw(:all);
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Portal::Base Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '0.04';

=head1 NAME

Main - Object used to build a Main Object Class.

=head1 SYNOPSIS

  use Portal::Accounting::DB::Main;
  my $obj = Portal::Accounting::DB::Main->new;
  if ($obj->didErrorOccur())
  {
    die $obj->errorMessage();
  }

=head1 DESCRIPTION

Main is a object used to retreive data from the database.

=head1 Exported FUNCTIONS

B<NOTE>: I<bool> = 1(true), 0(false)

=over 4

=item scalar new(dbHandle)

 Creates a new instance of the Portal::Accounting::DB::Main object.
 See Portal::Base(3) for a listing of required arguments.

 requires:
   dbHandle

=cut

sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  my %args = ( dbHandle => undef, @_ );

  if ($self->didErrorOccur)
  {
    $self->prefixError();
    return $self;
  }

  # instantiate anything unique to this module
  $self->{portalVariables} = Portal::Data::Variables->new(langObj => $self->{langObj});
  if ($self->{portalVariables}->didErrorOccur)
  {
    $self->setError($self->{portalVariables}->errorMessage);
    return $self;
  }
  eval { $self->{appConfigObj} = Portal::Accounting::Data::Config->new(langObj => $self->{langObj}); };
  if ($@)
  {
    $self->setError($@);
    return $self;
  }
  if ($self->{appConfigObj}->didErrorOccur)
  {
    $self->setError($self->{appConfigObj}->errorMessage);
    return $self;
  }
  $self->{dbHandle} = $args{dbHandle};

  $self->{methods} = Portal::Methods->new(langObj => $self->{langObj});

  # do validation
  if (!$self->Portal::Accounting::DB::Main::isValid)
  {
    # the error is set in the isValid() method.
    return $self;
  }

  # do anything else you might need to do.

  return $self;
}

=item bool isValid(void)

 Returns 0 or 1 to indicate if the object is valid.
 The error will be available via errorMessage().

=cut

sub isValid
{
  my $self = shift;

  # make sure our Parent class is valid.
  if (!$self->SUPER::isValid())
  {
    $self->prefixError();
    return 0;
  }

  # validate our parameters.
  if (!defined $self->{dbHandle})
  {
    $self->missing("dbHandle");
  }

  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->postfixError($self->genErrorString("all"));
    return 0;
  }

  return 1;
}

#******** Helper functions ***********

=item bool areWeConfigured(void)

    Returns 1 if we are configured, 0 if not, -1 on error.

=cut
sub areWeConfigured
{
  my $self = shift;
  my $result = 0;  # startout saying we aren't configured.
  my $sth;
  my @info;

  $sth = $self->{dbHandle}->read(sql => "SELECT never_run FROM version_tb");
  if ($self->{dbHandle}->didErrorOccur)
  {
    $self->{error} = 1;
    $self->{errorString} = $self->{dbHandle}->errorMessage;
    return -1;
  }
  @info = $sth->fetchrow_array;
  if (!defined $info[0])
  {
    $self->{error} = 1;
    $self->{errorString} = "nothing returned!<br>\n";
    return -1;
  }
  else
  {
    $result = !$info[0];
  }

  return $result;
}

=item bool sayWeAreConfigured(void)

    Returns 1 on success, 0 on error.

=cut
sub sayWeAreConfigured
{
  my $self = shift;

  $self->{dbHandle}->write(sql => "UPDATE version_tb SET never_run = '0'");  # set it false.  Looking ahead to days of MySQL possibility.
  if ($self->{dbHandle}->didErrorOccur)
  {
    $self->{error} = 1;
    $self->{errorString} = $self->{dbHandle}->errorMessage;
    return 0;
  }
  $self->{dbHandle}->commit;  # make change permanent.

  return 1;
}

=item int getAccountLength(void)

    Returns the value of acc_digits from version_tb.

=cut
sub getAccountLength
{
  my $self = shift;
  my $length = 0;
  my $sth;
  my @info;

  $sth = $self->{dbHandle}->read(sql => "SELECT acc_digits FROM version_tb");
  if ($self->{dbHandle}->didErrorOccur)
  {
    $self->{error} = 1;
    $self->{errorString} = $self->{dbHandle}->errorMessage;
    return 0;
  }
  @info = $sth->fetchrow_array;
  if (!defined $info[0])
  {
    $self->{error} = 1;
    $self->{errorString} = "acc_digits not defined in database!<br>\n";
    return 0;
  }
  $length = $info[0];

  return $length;
}

=item int getSubAccountLength(void)

    Returns the value of sub_acc_digits from version_tb.

=cut
sub getSubAccountLength
{
  my $self = shift;
  my $length = 0;
  my $sth;
  my @info;

  $sth = $self->{dbHandle}->read(sql => "SELECT sub_acc_digits FROM version_tb");
  if ($self->{dbHandle}->didErrorOccur)
  {
    $self->{error} = 1;
    $self->{errorString} = $self->{dbHandle}->errorMessage;
    return 0;
  }
  @info = $sth->fetchrow_array;
  if (!defined $info[0])
  {
    $self->{error} = 1;
    $self->{errorString} = "sub_acc_digits not defined in database!<br>\n";
    return 0;
  }
  $length = $info[0];

  return $length;
}

=item AccountObject[] getListOfAccounts(type, active, taxable, location, ledgerCode, orderBy)

    Returns an array of AccountObjects that represent the requested
    data.  location can be (global or local).  If location = local,
    then you must specify the ledgerCode to work with.  If type,
    active and taxable are not specified then they are not used in
    the query.  Result is sorted by type then number unless
    specified in the orderBy string.

=cut
sub getListOfAccounts
{
  my $self = shift;
  my %args = ( location => "global", orderBy => "type, number", @_ );
  my $location = $args{location};
  my $type = (exists $args{type} ? $args{type} : -1);
  my $taxable = (exists $args{taxable} ? $args{taxable} : -1);
  my $active = (exists $args{active} ? $args{active} : -1);
  my $ledgerCode = (exists $args{ledgerCode} ? $args{ledgerCode} : -1);
  my $orderBy = $args{orderBy};
  my @Accounts = ();
  my $sth;
  my @info;

  # validate input
  if ($location !~ /^(global|local)$/)
  {
    $self->{error} = 1;
    $self->{errorString} = "location = '$location' is invalid!<br>\n";
    return @Accounts;
  }
  if ($location eq "local" && $ledgerCode == -1)
  {
    $self->{error} = 1;
    $self->{errorString} = "ledgerCode must be specified when gathering Local Account Entries!<br>\n";
    return @Accounts;
  }
  if ($type != -1 && $type < -1)
  {
    $self->{error} = 1;
    $self->{errorString} = "type = '$type' is invalid!<br>\n";
    return @Accounts;
  }
  if ($taxable != -1 && !exists $self->{portalVariables}->{falseTrue}->{$taxable})
  {
    $self->{error} = 1;
    $self->{errorString} = "taxable = '$taxable' is invalid!<br>\n";
    return @Accounts;
  }
  if ($active != -1 && !exists $self->{portalVariables}->{falseTrue}->{$active})
  {
    $self->{error} = 1;
    $self->{errorString} = "active = '$active' is invalid!<br>\n";
    return @Accounts;
  }
  if (length $orderBy == 0)
  {
    $self->{error} = 1;
    $self->{errorString} = "orderBy = '$orderBy' is invalid!<br>\n";
    return @Accounts;
  }

  # get the minAccountLength for this system
  my $minAccountLength = $self->getAccountLength;
  if ($self->didErrorOccur)
  {
    $self->{error} = 1;
    $self->{errorString} = $self->errorMessage;
    return @Accounts;
  }

  my $sql = "";
  if ($location eq "global")
  {
    $sql = "SELECT name, number, type, active, taxable FROM account_tb";
    my $where = 0;
    if ($type != -1)
    {
      if (!$where)
      {
        $sql .= " WHERE ";
        $where = 1;
      }
      $sql .= "type = '$type'";
    }
    if ($active != -1)
    {
      if (!$where)
      {
        $sql .= " WHERE ";
        $where = 1;
      }
      else
      {
        $sql .= " AND ";
      }
      $sql .= "active = '$active'";
    }
    if ($taxable != -1)
    {
      if (!$where)
      {
        $sql .= " WHERE ";
        $where = 1;
      }
      else
      {
        $sql .= " AND ";
      }
      $sql .= "taxable = '$taxable'";
    }
    $sql .= " ORDER BY $orderBy";
  }
  else  # local accounts table.
  {
    $sql = "SELECT a.name, a.number, a.type, a.active, a.taxable FROM account_tb AS a, local_accounts_tb WHERE local_accounts_tb.ledger_code = '$ledgerCode' AND number = local_accounts_tb.acct_num";
    if ($type != -1)
    {
      $sql .= " AND type = '$type'";
    }
    if ($active != -1)
    {
      $sql .= " AND active = '$active'";
    }
    if ($taxable != -1)
    {
      $sql .= " AND taxable = '$taxable'";
    }
    $sql .= " ORDER BY $orderBy";
  }
  # now execute the sql statement
  $sth = $self->{dbHandle}->read(sql => $sql);
  if ($self->{dbHandle}->didErrorOccur)
  {
    $self->{error} = 1;
    $self->{errorString} = $self->{dbHandle}->errorMessage;
    return @Accounts;
  }
  while (@info = $sth->fetchrow_array)
  {
    my $accountObj = Portal::Accounting::Objects::AccountObject->new(name => $info[0], number => $info[1], type => $info[2],
                     active => $self->{portalVariables}->{trueFalse}->{$info[3]}, creating => "false",
                     taxable => $self->{portalVariables}->{trueFalse}->{$info[4]}, minAccountLength => $minAccountLength, langObj => $self->{langObj});
    if ($accountObj->didErrorOccur)
    {
      $self->{error} = 1;
      $self->{errorString} = $accountObj->errorMessage;
      return @Accounts;
    }
    $Accounts[++$#Accounts] = $accountObj;
  }

  return @Accounts;
}

=item SubAccountObject[] getListOfSubAccounts(type, orderBy)

    Returns an array of SubAccountObjects that represent the requested data.
    If type is not specified then it is not used in the query.
    Type may be a comma separated list.
    Result is sorted by type then sub_acc_number unless specified in the orderBy string.

=cut
sub getListOfSubAccounts
{
  my $self = shift;
  my %args = ( orderBy => "type, sub_acc_number", @_ );
  my $type = (exists $args{type} ? $args{type} : "");
  my $orderBy = $args{orderBy};
  my @SubAccounts = ();
  my $sth;
  my $typeString = "";
  my @info;

  # validate input
  if (length $orderBy == 0)
  {
    $self->{error} = 1;
    $self->{errorString} = "orderBy = '$orderBy' is invalid!<br>\n";
    return @SubAccounts;
  }

  # get the minSubAccountLength for this system
  my $minSubAccountLength = $self->getSubAccountLength;
  if ($self->didErrorOccur)
  {
    $self->{error} = 1;
    $self->{errorString} = $self->errorMessage;
    return @SubAccounts;
  }

  my $index = 0;
  foreach my $temp (split ",", $type)
  {
    if ($index++ > 0)
    {
      $typeString .= " OR";
    }
    $typeString .= " type = '$temp'";
  }
  my $sql = "SELECT sub_acc_name, sub_acc_number, type FROM sub_acc_tb";
  if ($type != "")
  {
    $sql .= " WHERE $typeString";
  }
  $sql .= " ORDER BY $orderBy";

  # now execute the sql statement
  $sth = $self->{dbHandle}->read(sql => $sql);
  if ($self->{dbHandle}->didErrorOccur)
  {
    $self->{error} = 1;
    $self->{errorString} = $self->{dbHandle}->errorMessage;
    return @SubAccounts;
  }
  while (@info = $sth->fetchrow_array)
  {
    my $subAccountObj = Portal::Accounting::Objects::SubAccountObject->new(name => $info[0],
                        number => $info[1], type => $info[2], minSubAccountLength => $minSubAccountLength,
                        creating => "false", langObj => $self->{langObj});
    if ($subAccountObj->didErrorOccur)
    {
      $self->{error} = 1;
      $self->{errorString} = $subAccountObj->errorMessage;
      return @SubAccounts;
    }
    $SubAccounts[++$#SubAccounts] = $subAccountObj;
  }

  return @SubAccounts;
}

# getListOfSubEntities
# required args:
# optional: ledgerCode, orderBy (name, code)
# returns: Array of SubEntityObjects
# This routine selects all sub_entities for a given ledger.
# The orderBy string must be a valid ORDER BY possible statement for this query.
sub getListOfSubEntities
{
  my $self = shift;
  my %args = ( orderBy => "name", @_ );
  my $orderBy = $args{orderBy};
  my $ledgerCode = $args{ledgerCode};
  my @SubEntitys = ();
  my $sth;
  my @info;

  # validate input
  if (length $orderBy == 0)
  {
    $self->{error} = 1;
    $self->{errorString} = "orderBy = '$orderBy' is invalid!<br>\n";
    return @SubEntitys;
  }

  my $sql = "SELECT name, code, ledger_code FROM sub_entity_tb";
  if ($ledgerCode ne "")
  {
    $sql .= " WHERE ledger_code = '$ledgerCode'";
  }
  $sql .= " ORDER BY $orderBy";

  # now execute the sql statement
  $sth = $self->{dbHandle}->read(sql => $sql);
  if ($self->{dbHandle}->didErrorOccur)
  {
    $self->{error} = 1;
    $self->{errorString} = $self->{dbHandle}->errorMessage;
    return @SubEntitys;
  }
  while (@info = $sth->fetchrow_array)
  {
    my $SubEntityObj = Portal::Accounting::Objects::SubEntityObject->new(name => $info[0], code => $info[1], ledgerCode =>$info[2], langObj => $self->{langObj});
    if ($SubEntityObj->didErrorOccur)
    {
      $self->{error} = 1;
      $self->{errorString} = $SubEntityObj->errorMessage;
      return @SubEntitys;
    }
    $SubEntitys[++$#SubEntitys] = $SubEntityObj;
  }

  return @SubEntitys;
}

=item AccountObject getAccountInfo(name, number)

    takes: name or number
    returns: AccountObject or undef if not found

=cut
sub getAccountInfo
{
  my $self = shift;
  my %args = ( @_ );
  my $type = "";
  my $sth;
  my @info;

  if (exists $args{name})
  {
    $type = "name";
    if (length $args{$type} == 0)
    {
      $self->{error} = 1;
      $self->{errorString} = "name must be specified!<br>\n";
      return undef;
    }
  }
  elsif (exists $args{number})
  {
    $type = "number";
    if (length $args{$type} == 0 || $args{$type} < 0)
    {
      $self->{error} = 1;
      $self->{errorString} = "number = '$args{$type}' is invalid!<br>\n";
      return undef;
    }
  }
  else
  {
    $self->{error} = 1;
    $self->{errorString} = "You must specify either number or name!<br>\n";
    return undef;
  }
  my $accountObj;

  # get the minAccountLength for this system
  my $minAccountLength = $self->getAccountLength;
  if ($self->didErrorOccur)
  {
    $self->{error} = 1;
    $self->{errorString} = $self->errorMessage;
    return undef;
  }

  # now we select this account from the system and return the object.
  $sth = $self->{dbHandle}->read(sql => "SELECT name, number, type, active, taxable FROM account_tb WHERE $type = ?",
                                    plug => [ $args{$type} ]);
  if ($self->{dbHandle}->didErrorOccur)
  {
    $self->{error} = 1;
    $self->{errorString} = $self->{dbHandle}->errorMessage;
    return undef;
  }
  @info = $sth->fetchrow_array;
  if (defined $info[0])
  {
    $accountObj = Portal::Accounting::Objects::AccountObject->new(name => $info[0], number => $info[1], type => $info[2],
             active => $self->portalVariables->trueFalse->{$info[3]}, taxable => $self->portalVariables->trueFalse->{$info[4]},
             minAccountLength => $minAccountLength, creating => "false", langObj => $self->{langObj});
    if ($accountObj->didErrorOccur)
    {
      $self->setError(errorString => $accountObj->errorMessage);
      return undef;
    }
  }
  else
  {
    $accountObj = undef;
  }

  return $accountObj;
}

# getSubAccountInfo
# takes: name or number
# returns: SubAccountObject or undef if not found
sub getSubAccountInfo
{
  my $self = shift;
  my %args = ( @_ );
  my $type = "";
  my $sth;
  my @info;

  if (exists $args{name})
  {
    $type = "name";
    if (length $args{$type} == 0)
    {
      $self->{error} = 1;
      $self->{errorString} = "name must be specified!<br>\n";
      return undef;
    }
  }
  elsif (exists $args{number})
  {
    $type = "number";
    if (length $args{$type} == 0 || $args{$type} < 0)
    {
      $self->{error} = 1;
      $self->{errorString} = "number = '$args{$type}' is invalid!<br>\n";
      return undef;
    }
  }
  else
  {
    $self->{error} = 1;
    $self->{errorString} = "You must specify either number or name!<br>\n";
    return undef;
  }
  my $subAccountObj;

  # get the minSubAccountLength for this system
  my $minSubAccountLength = $self->getSubAccountLength;
  if ($self->didErrorOccur)
  {
    $self->{error} = 1;
    $self->{errorString} = $self->errorMessage;
    return undef;
  }

  my $realType = ( $type eq "name" ? "sub_acc_name" : "sub_acc_number");

  # now we select this account from the system and return the object.
  $sth = $self->{dbHandle}->read(sql => "SELECT sub_acc_name, sub_acc_number, type FROM sub_acc_tb WHERE $realType = ?",
                                    plug => [ $args{$type} ]);
  if ($self->{dbHandle}->didErrorOccur)
  {
    $self->{error} = 1;
    $self->{errorString} = $self->{dbHandle}->errorMessage;
    return undef;
  }
  @info = $sth->fetchrow_array;
  if (defined $info[0])
  {
    $subAccountObj = Portal::Accounting::Objects::SubAccountObject->new(name => $info[0], number => $info[1], type => $info[2],
                     minSubAccountLength => $minSubAccountLength, creating => "false", langObj => $self->{langObj});
    if ($subAccountObj->didErrorOccur)
    {
      $self->setError(errorString => $subAccountObj->errorMessage);
      return undef;
    }
  }
  else
  {
    $subAccountObj = undef;
  }

  return $subAccountObj;
}

# logTempFile
# requires: filename
# optional:
# returns:
# This routine makes an entry in the database that indicates a tempfile that needs to be cleaned up later by the cron job.
sub logTempFile
{
  my $self = shift;
  my %args = ( filename => "", @_ );
  my $filename = $args{filename};
  my $sth;
  my @info;

  if (length $filename == 0)
  {
    $self->{error} = 1;
    $self->{errorString} = "filename = '$filename' is invalid!<br>\n";
    return;
  }
  if (! -e $filename)
  {
    $self->{error} = 1;
    $self->{errorString} = "filename = '$filename' does not exist!<br>\n";
    return;
  }

  $sth = $self->{dbHandle}->write(sql => "INSERT INTO temp_file_tb (timeof, fname) VALUES (now(), ?)", plug => [ $filename ]);
  if ($self->{dbHandle}->didErrorOccur)
  {
    $self->{error} = 1;
    $self->{errorString} = $self->{dbHandle}->errorMessage;
    return;
  }
  $self->{dbHandle}->commit;
}

=item FiscalYearObject getCurrentFiscalYear()

     requires: ledgerCode
     optional:
     returns: FiscalYearObject or undef if error
    summary: This method returns a FiscalYearObject that holds the current Fiscal Year's info for the specified ledger.
=cut
sub getCurrentFiscalYear
{
  my $self = shift;
  my %args = ( @_ );
  my $ledgerCode = $args{ledgerCode};
  my $fiscalYearObj = undef;
  my $sth;
  my @info;

  if (length $ledgerCode == 0)
  {
    $self->{error} = 1;
    $self->{errorString} = "ledgerCode = '$ledgerCode' is invalid!<br>\n";
    return $fiscalYearObj;
  }

  $sth = $self->{dbHandle}->read(sql => "SELECT year, closed, start_date, end_date, q1_start, q1_end, q2_start, q2_end, q3_start, q3_end, q4_start, q4_end, ledger_code FROM known_years_tb WHERE ledger_code = ? AND year = (SELECT working_year FROM variables_tb WHERE ledger_code = ?)",
                                    plug => [ $ledgerCode, $ledgerCode ]);
  if ($self->{dbHandle}->didErrorOccur)
  {
    $self->{error} = 1;
    $self->{errorString} = $self->{dbHandle}->errorMessage;
    return $fiscalYearObj;
  }
  @info = $sth->fetchrow_array;
  if (!defined $info[0])
  {
    $self->{error} = 1;
    $self->{errorString} = "No data returned!<br>\n";
    return $fiscalYearObj;
  }
  $fiscalYearObj = Portal::Accounting::Objects::FiscalYearObject->new(year => $info[0], closed => $self->portalVariables->trueFalse->{$info[1]},
                   startDate => $info[2], endDate => $info[3], q1Start => $info[4], q1End => $info[5], q2Start => $info[6], q2End => $info[7],
                   q3Start => $info[8], q3End => $info[9], q4Start => $info[10], q4End => $info[11], ledgerCode => $info[12], langObj => $self->{langObj});
  if ($fiscalYearObj->didErrorOccur)
  {
    $self->{error} = 1;
    $self->{errorString} = $fiscalYearObj->errorMessage;
    return undef;
  }

  return $fiscalYearObj;
}

# getAllKnownYears
# requires: ledgerCode
# optional:
# returns: array of FiscalYearObjects or undef if error
# summary: This method returns an array of all Fiscal Years for the specified ledger in oldest to newest order.
sub getAllKnownYears
{
  my $self = shift;
  my %args = ( @_ );
  my $ledgerCode = $args{ledgerCode};
  my @knownYears = ();
  my $sth;
  my @info;

  if (length $ledgerCode == 0)
  {
    $self->{error} = 1;
    $self->{errorString} = "ledgerCode = '$ledgerCode' is invalid!<br>\n";
    return @knownYears;
  }

  $sth = $self->{dbHandle}->read(sql => "SELECT year, closed, start_date, end_date, q1_start, q1_end, q2_start, q2_end, q3_start, q3_end, q4_start, q4_end, ledger_code FROM known_years_tb WHERE ledger_code = ? ORDER BY year",
                                    plug => [ $ledgerCode ]);
  if ($self->{dbHandle}->didErrorOccur)
  {
    $self->{error} = 1;
    $self->{errorString} = $self->{dbHandle}->errorMessage;
    return @knownYears;
  }

  while (@info = $sth->fetchrow_array)
  {
    my $fiscalYearObj = Portal::Accounting::Objects::FiscalYearObject->new(year => $info[0], closed => $self->portalVariables->trueFalse->{$info[1]},
                     startDate => $info[2], endDate => $info[3], q1Start => $info[4], q1End => $info[5], q2Start => $info[6], q2End => $info[7],
                     q3Start => $info[8], q3End => $info[9], q4Start => $info[10], q4End => $info[11], ledgerCode => $info[12], langObj => $self->{langObj});
    if ($fiscalYearObj->didErrorOccur)
    {
      $self->{error} = 1;
      $self->{errorString} = $fiscalYearObj->errorMessage;
      return undef;
    }
    $knownYears[++$#knownYears] = $fiscalYearObj;
  }
  if (scalar @knownYears == 0)
  {
    $self->{error} = 1;
    $self->{errorString} = "No data returned!<br>\n";
    return @knownYears;
  }

  return @knownYears;
}

# getDBVersion
# requires:
# optional:
# returns: Database version or "" if error
sub getDBVersion
{
  my $self = shift;
  my $version = "";
  my $sth;
  my @info;

  $sth = $self->{dbHandle}->read(sql => "SELECT version FROM version_tb");
  if ($self->{dbHandle}->didErrorOccur)
  {
    $self->{error} = 1;
    $self->{errorString} = $self->{dbHandle}->errorMessage;
    return $version;
  }
  @info = $sth->fetchrow_array;
  if (!defined $info[0])
  {
    $self->{error} = 1;
    $self->{errorString} = "Nothing returned from database!<br>\n";
    return $version;
  }
  $version = $info[0];

  return $version;
}

# setDBVersion
# requires: version
# optional:
# returns: 1=OK, 0=Error
sub setDBVersion
{
  my $self = shift;
  my %args = ( version => "", @_ );
  my $version = $args{version};

  if ($version !~ /^\d+\.\d+$/)
  {
    $self->{error} = 1;
    $self->{errorString} = "version = '$version' is invalid!<br>\n";
    return 0;
  }

  my $rc = $self->{dbHandle}->write(sql => "UPDATE version_tb SET version = ?", plug => [$version]);
  if ($self->{dbHandle}->didErrorOccur)
  {
    $self->{error} = 1;
    $self->{errorString} = $self->{dbHandle}->errorMessage;
    return 0;
  }
  if ($rc == 0)
  {
    $self->{error} = 1;
    $self->{errorString} = "Update of version didn't happen!<br>\n";
    return 0;
  }
  $self->{dbHandle}->commit;  # commit changes.

  return 1;
}

# getTempFilesToCleanup
# requires: timeLength
# optional: timeValue (hours, minutes, seconds, etc.)
# returns: Array of files that need to be removed.
# summary: This method checks the temp_file_tb to see if there are any records that are older than the specified timeLength and if so,
# it returns the list of them in the array.
sub getTempFilesToCleanup
{
  my $self = shift;
  my %args = ( timeLength => 3, timeValue => "hours", @_ );
  my $timeLength = $args{timeLength};
  my $timeValue = $args{timeValue};
  my @files = ();
  my $sth;
  my @info;

  # validate the timeLength and timeValue values
  if ($timeLength < 1)
  {
    $self->{error} = 1;
    $self->{errorString} = "timeLength = '$timeLength' is invalid!<br>\n";
    return @files;
  }
  if ($timeValue !~ /^(hours|minutes|seconds|days|months|years)$/)
  {
    $self->{error} = 1;
    $self->{errorString} = "timeValue = '$timeValue' is invalid!<br>\n";
    return @files;
  }

  $sth = $self->{dbHandle}->read(sql => "SELECT fname FROM temp_file_tb WHERE timeof < CAST(now() - CAST('$timeLength $timeValue' AS INTERVAL) AS TIMESTAMP)");
  if ($self->{dbHandle}->didErrorOccur)
  {
    $self->{error} = 1;
    $self->{errorString} = $self->{dbHandle}->errorMessage;
    return @files;
  }

  while (@info = $sth->fetchrow_array)
  {
    $files[++$#files] = $info[0];
  }

  return @files;
}

# cleanupTempFile
# requires: fileName
# optional:
# returns: 1=OK, 0=Error, -1=Doesn't exist on disk (but delete from db was ok), -2=Doesn't exist in db (but did exist on disk and was deleted),
#         -3=Doesn't exist on disk or in db.
# summary: Deletes the specified file and removes it from the temp_file_tb table.
sub cleanupTempFile
{
  my $self = shift;
  my %args = ( fileName => "", @_ );
  my $file = $args{fileName};
  my $result = 1;

  if (length $file == 0)
  {
    $self->{error} = 1;
    $self->{errorString} = "file must be specified!<br>\n";
    return 0;
  }
  if (! -e $file)
  {
    $result = -1;
  }
  unlink $file;

  my $rc = $self->{dbHandle}->write(sql => "DELETE FROM temp_file_tb WHERE fname = ?", plug => [$file]);
  if ($self->{dbHandle}->didErrorOccur)
  {
    $self->{error} = 1;
    $self->{errorString} = $self->{dbHandle}->errorMessage;
    return 0;
  }
  if ($rc == 0)
  {
    if ($result == -1)
    {
      $result = -3;  # signal it doesn't exist on disk or in db.
    }
    else
    {
      $result = -2; # it doesn't exist in the db.
    }
  }
  else
  {
    $self->{dbHandle}->commit;
  }

  return $result;
}

# getNextAvailableAccountNumber
# requires: firstDigit, minLength
# optional:
# returns: The currently high value + 1 that fits the specified requirements or -1 if error.
# summary: Queries the database for the max(number) where the number like 'firstDigit%' and length (number) = minLength.
# If nothing returned then we make a number firstDigit (minLength-1) 0's + 1 and return it, else we return the max value + 1.
sub getNextAvailableAccountNumber
{
  my $self = shift;
  my %args = ( firstDigit => '1', minLength => '4', @_ );
  my $firstDigit = $args{firstDigit};
  my $minLength = $args{minLength};
  my $result = -1;
  my $sth;
  my @info;

  if ($firstDigit < 1 || $firstDigit > 5)
  {
    $self->{error} = 1;
    $self->{errorString} = "firstDigit = '$firstDigit' is invalid!  It must be between 1 and 5.<br>\n";
    return $result;
  }
  if ($minLength < $self->{appConfigObj}->AccountLengthMin || $minLength > $self->{appConfigObj}->AccountLengthMax)
  {
    $self->{error} = 1;
    $self->{errorString} = "minLength = '$minLength' is invalid!  It must be between $self->{appConfigObj}->{AccountLengthMin} and $self->{appConfigObj}->{AccountLengthMax}.<br>\n";
    return $result;
  }

  $sth = $self->{dbHandle}->read(sql => "SELECT MAX(number) FROM account_tb WHERE number LIKE '$firstDigit\%' AND LENGTH (number) = $minLength");
  if ($self->{dbHandle}->didErrorOccur)
  {
    $self->{error} = 1;
    $self->{errorString} = $self->{dbHandle}->errorMessage;
    return $result;
  }
  @info = $sth->fetchrow_array;
  if (!defined $info[0])
  {
    # we have to make up a value.
    $result = $firstDigit . "0" x ($minLength - 1);
    $result += 1;
  }
  else
  {
    # take the value + 1 and return it.
    $result = $info[0] + 1;
  }

  return $result;
}

# getNextAvailableSubAccountNumber
# requires: minLength
# optional:
# returns: next highest number in the specified length + 1 or -1 on error.
# summary: Queries the database for the max account number where length (number) = minLength.
# If nothing is returned, then we make a number 1 + (minLength - 1) 0's + 1 and return it, else we return
# the max value + 1.
sub getNextAvailableSubAccountNumber
{
  my $self = shift;
  my %args = ( minLength => '4', @_ );
  my $minLength = $args{minLength};
  my $result = -1;
  my $sth;
  my @info;

  if ($minLength < $self->{appConfigObj}->SubAccountLengthMin || $minLength > $self->{appConfigObj}->SubAccountLengthMax)
  {
    $self->{error} = 1;
    $self->{errorString} = "minLength = '$minLength' is invalid!  It must be between $self->{appConfigObj}->{SubAccountLengthMin} and $self->{appConfigObj}->{SubAccountLengthMax}.<br>\n";
    return $result;
  }

  $sth = $self->{dbHandle}->read(sql => "SELECT MAX(sub_acc_number) FROM sub_acc_tb WHERE LENGTH (sub_acc_number) = $minLength");
  if ($self->{dbHandle}->didErrorOccur)
  {
    $self->{error} = 1;
    $self->{errorString} = $self->{dbHandle}->errorMessage;
    return $result;
  }
  @info = $sth->fetchrow_array;
  if (!defined $info[0])
  {
    # we have to make up a value.
    $result = "1" . "0" x ($minLength - 1);
    $result += 1;
  }
  else
  {
    # take the value + 1 and return it.
    $result = $info[0] + 1;
  }

  return $result;
}

# createAccount
# requires: accountObj
# optional:
# returns: 1=OK, 0=Error, -1=Both Name and Number Already exists, -2=Name Already exists, -3=Number Already exists
#          second element of returned array contains the type of the account found, if error; third/fourth element(s)
#          contain(s) the value(s) not included in the SQL statement for a match (i.e., name only will return type, number;
#          number only will return type, name; name and number will only return type)
# summary: creates the specified account in the database.
sub createAccount
{
  my $self = shift;
  my %args = ( accountObj => undef, @_ );
  my $accountObj = $args{accountObj};
  my $sth;
  my @info;

  if (!defined $accountObj)
  {
    $self->{error} = 1;
    $self->{errorString} = "accountObj must be defined!<br>\n";
    return (0);
  }

  $sth = $self->{dbHandle}->read(sql => "SELECT type, taxable FROM account_tb WHERE name = ? AND number = ?",
                                    plug => [ $accountObj->name, $accountObj->number ]);
  if ($self->{dbHandle}->didErrorOccur)
  {
    $self->{error} = 1;
    $self->{errorString} = $self->{dbHandle}->errorMessage;
    return (0);
  }
  @info = $sth->fetchrow_array;
  if (defined $info[0])
  {
    return (-1, $info[0], $info[1]);  # both name and number already exist in the database!
  }
  $sth = $self->{dbHandle}->read(sql => "SELECT number, type, taxable FROM account_tb WHERE name = ?",
                                    plug => [ $accountObj->name ]);
  if ($self->{dbHandle}->didErrorOccur)
  {
    $self->{error} = 1;
    $self->{errorString} = $self->{dbHandle}->errorMessage;
    return (0);
  }
  @info = $sth->fetchrow_array;
  if (defined $info[0])
  {
    return (-2, $info[1], $info[2], $info[0]);  # name already exists in the database!
  }
  $sth = $self->{dbHandle}->read(sql => "SELECT name, type, taxable FROM account_tb WHERE number = ?",
                                    plug => [ $accountObj->number ]);
  if ($self->{dbHandle}->didErrorOccur)
  {
    $self->{error} = 1;
    $self->{errorString} = $self->{dbHandle}->errorMessage;
    return (0);
  }
  @info = $sth->fetchrow_array;
  if (defined $info[0])
  {
    return (-3, $info[1], $info[2], $info[0]);  # number already exists in the database!
  }

  my $rc = $self->{dbHandle}->write(sql => "INSERT INTO account_tb (name, number, type, active, taxable) VALUES (?, ?, ?, ?, ?)",
                                    plug => [ $accountObj->name, $accountObj->number, $accountObj->type, $accountObj->active, $accountObj->taxable ]);
  if ($self->{dbHandle}->didErrorOccur)
  {
    $self->{error} = 1;
    $self->{errorString} = $self->{dbHandle}->errorMessage;
    return (0);
  }
  $self->{dbHandle}->commit;

  return (1);
}

# createSubAccount
# requires: subAccountObj
# optional:
# returns: 1=OK, 0=Error, -1=Both Name and Number Already exists, -2=Name Already exists, -3=Number Already exists
#          second element of returned array contains the type of the account found, if error; third/fourth element(s)
#          contain(s) the value(s) not included in the SQL statement for a match (i.e., name only will return type, number;
#          number only will return type, name; name and number will only return type)
# summary: creates the specified sub account in the database.
sub createSubAccount
{
  my $self = shift;
  my %args = ( subAccountObj => undef, @_ );
  my $subAccountObj = $args{subAccountObj};
  my $sth;
  my @info;

  if (!defined $subAccountObj)
  {
    $self->{error} = 1;
    $self->{errorString} = "subAccountObj must be defined!<br>\n";
    return (0);
  }

  $sth = $self->{dbHandle}->read(sql => "SELECT type FROM sub_acc_tb WHERE sub_acc_name = ? AND sub_acc_number = ?",
                                    plug => [  $subAccountObj->name, $subAccountObj->number ]);
  if ($self->{dbHandle}->didErrorOccur)
  {
    $self->{error} = 1;
    $self->{errorString} = $self->{dbHandle}->errorMessage;
    return (0);
  }
  @info = $sth->fetchrow_array;
  if (defined $info[0])
  {
    return (-1, $info[0]);  # both name and number already exist in the database!
  }
  $sth = $self->{dbHandle}->read(sql => "SELECT sub_acc_number, type FROM sub_acc_tb WHERE sub_acc_name = ?",
                                    plug => [ $subAccountObj->name ]);
  if ($self->{dbHandle}->didErrorOccur)
  {
    $self->{error} = 1;
    $self->{errorString} = $self->{dbHandle}->errorMessage;
    return (0);
  }
  @info = $sth->fetchrow_array;
  if (defined $info[0])
  {
    return (-2, $info[1], $info[0]);  # name already exists in the database!
  }
  $sth = $self->{dbHandle}->read(sql => "SELECT sub_acc_name, type FROM sub_acc_tb WHERE sub_acc_number = ?",
                                    plug => [ $subAccountObj->number ]);
  if ($self->{dbHandle}->didErrorOccur)
  {
    $self->{error} = 1;
    $self->{errorString} = $self->{dbHandle}->errorMessage;
    return (0);
  }
  @info = $sth->fetchrow_array;
  if (defined $info[0])
  {
    return (-3, $info[1], $info[0]);  # number already exists in the database!
  }

  my $rc = $self->{dbHandle}->write(sql => "INSERT INTO sub_acc_tb (sub_acc_name, sub_acc_number, type) VALUES (?, ?, ?)",
                                    plug => [ $subAccountObj->name, $subAccountObj->number, $subAccountObj->type ]);
  if ($self->{dbHandle}->didErrorOccur)
  {
    $self->{error} = 1;
    $self->{errorString} = $self->{dbHandle}->errorMessage;
    return (0);
  }
  $self->{dbHandle}->commit;

  return (1);
}

# assignAccountToLedger
# requires: accountObj, ledgerCode
# optional:
# returns: 1=OK, 0=Error, -1=Already exists
# summary: assigns the specified account to the specified ledger.
sub assignAccountToLedger
{
  my $self = shift;
  my %args = ( accountObj => undef, ledgerCode => "", @_ );
  my $accountObj = $args{accountObj};
  my $ledgerCode = $args{ledgerCode};
  my $sth;
  my @info;

  if (!defined $accountObj)
  {
    $self->{error} = 1;
    $self->{errorString} = "accountObj must be defined!<br>\n";
    return 0;
  }

  if (length $ledgerCode == 0)
  {
    $self->{error} = 1;
    $self->{errorString} = "ledgerCode must be defined!<br>\n";
    return 0;
  }

  $sth = $self->{dbHandle}->read(sql => "SELECT acct_num, ledger_code FROM local_accounts_tb WHERE acct_num = ? AND ledger_code = ?",
                                    plug => [ $accountObj->number, $ledgerCode ]);
  if ($self->{dbHandle}->didErrorOccur)
  {
    $self->{error} = 1;
    $self->{errorString} = $self->{dbHandle}->errorMessage;
    return 0;
  }
  @info = $sth->fetchrow_array;
  if (defined $info[0])
  {
    return -1;  # it already exists in the database!
  }

  my $rc = $self->{dbHandle}->write(sql => "INSERT INTO local_accounts_tb (acct_num, taxable, ledger_code) VALUES (?, ?, ?)",
                                    plug => [ $accountObj->number, $accountObj->taxable, $ledgerCode ]);
  if ($self->{dbHandle}->didErrorOccur)
  {
    $self->{error} = 1;
    $self->{errorString} = $self->{dbHandle}->errorMessage;
    return 0;
  }
  $self->{dbHandle}->commit;

  return 1;
}

# unAssignAccountFromLedger
# requires: accountObj, ledgerCode
# optional:
# returns: 1=OK, 0=Error, -1=Doesn't exist anymore
# summary: unassigns the specified account from the specified ledger.
sub unAssignAccountFromLedger
{
  my $self = shift;
  my %args = ( accountObj => undef, ledgerCode => "", @_ );
  my $accountObj = $args{accountObj};
  my $ledgerCode = $args{ledgerCode};
  my $sth;
  my @info;

  if (!defined $accountObj)
  {
    $self->{error} = 1;
    $self->{errorString} = "accountObj must be defined!<br>\n";
    return 0;
  }

  if (length $ledgerCode == 0)
  {
    $self->{error} = 1;
    $self->{errorString} = "ledgerCode must be defined!<br>\n";
    return 0;
  }

  $sth = $self->{dbHandle}->read(sql => "SELECT acct_num, ledger_code FROM local_accounts_tb WHERE acct_num = ? AND ledger_code = ?",
                                    plug => [ $accountObj->number, $ledgerCode ]);
  if ($self->{dbHandle}->didErrorOccur)
  {
    $self->{error} = 1;
    $self->{errorString} = $self->{dbHandle}->errorMessage;
    return 0;
  }
  @info = $sth->fetchrow_array;
  if (!defined $info[0])
  {
    return -1;  # it doesn't exist in the database!
  }

  my $rc = $self->{dbHandle}->write(sql => "DELETE FROM local_accounts_tb WHERE acct_num = ? AND ledger_code = ?",
                                    plug => [ $accountObj->number, $ledgerCode ]);
  if ($self->{dbHandle}->didErrorOccur)
  {
    $self->{error} = 1;
    $self->{errorString} = $self->{dbHandle}->errorMessage;
    return 0;
  }
  $self->{dbHandle}->commit;

  return 1;
}

# createStock
# requires: stockObj
# optional:
# returns: 1=OK, 0=Error, -1=Both Name, Ticker and Number Already exists, -2=Name Already exists, -3=Number Already exists
#          second element of returned array contains the type of the account found, if error; third/fourth element(s)
#          contain(s) the value(s) not included in the SQL statement for a match (i.e., name only will return type, number;
#          number only will return type, name; name and number will only return type), -4=Ticker Already exists.
# summary: creates the specified stock entry in the database.
sub createStock
{
  my $self = shift;
  my %args = ( stockObj => undef, @_ );
  my $stockObj = $args{stockObj};
  my $sth;
  my @info;

  if (!defined $stockObj)
  {
    $self->{error} = 1;
    $self->{errorString} = "stockObj must be defined!<br>\n";
    return (0);
  }

  $sth = $self->{dbHandle}->read(sql => "SELECT ticker FROM stocks_tb WHERE sub_acct = ? AND ticker = ? AND name = ?",
                                    plug => [  $stockObj->number, $stockObj->ticker, $stockObj->name ]);
  if ($self->{dbHandle}->didErrorOccur)
  {
    $self->{error} = 1;
    $self->{errorString} = $self->{dbHandle}->errorMessage;
    return (0);
  }
  @info = $sth->fetchrow_array;
  if (defined $info[0])
  {
    return (-1, $info[0]);  # both name, ticker and number already exist in the database!
  }
  $sth = $self->{dbHandle}->read(sql => "SELECT ticker, sub_acct FROM stocks_tb WHERE name = ?",
                                    plug => [ $stockObj->name ]);
  if ($self->{dbHandle}->didErrorOccur)
  {
    $self->{error} = 1;
    $self->{errorString} = $self->{dbHandle}->errorMessage;
    return (0);
  }
  @info = $sth->fetchrow_array;
  if (defined $info[0])
  {
    return (-2, $info[1], $info[0]);  # name already exists in the database!
  }
  $sth = $self->{dbHandle}->read(sql => "SELECT ticker, name FROM stocks_tb WHERE sub_acct = ?",
                                    plug => [ $stockObj->number ]);
  if ($self->{dbHandle}->didErrorOccur)
  {
    $self->{error} = 1;
    $self->{errorString} = $self->{dbHandle}->errorMessage;
    return (0);
  }
  @info = $sth->fetchrow_array;
  if (defined $info[0])
  {
    return (-3, $info[1], $info[0]);  # number already exists in the database!
  }
  # check for all tickers that are non-""
  if ($stockObj->ticker !~ /^\s*$/)
  {
    $sth = $self->{dbHandle}->read(sql => "SELECT name, sub_acct FROM stocks_tb WHERE ticker = ?",
                                      plug => [ $stockObj->ticker ]);
    if ($self->{dbHandle}->didErrorOccur)
    {
      $self->{error} = 1;
      $self->{errorString} = $self->{dbHandle}->errorMessage;
      return (0);
    }
    @info = $sth->fetchrow_array;
    if (defined $info[0])
    {
      return (-4, $info[0], $info[1]);  # ticker already exists in the database!
    }
  }

  my $rc = $self->{dbHandle}->write(sql => "INSERT INTO stocks_tb (ticker, name, sub_acct, market, class) VALUES (?, ?, ?, ?, ?)",
                                    plug => [ $stockObj->ticker, $stockObj->name, $stockObj->number, $stockObj->market, $stockObj->class ]);
  if ($self->{dbHandle}->didErrorOccur)
  {
    $self->{error} = 1;
    $self->{errorString} = $self->{dbHandle}->errorMessage;
    return (0);
  }
  $self->{dbHandle}->commit;

  return (1);
}

# deleteStock
# requires: number
# optional:
# returns: 1=OK, 0=Error, -1=Stock Sub Account No Longer exists
# summary: deletes the specified stock entry in the database.
sub deleteStock
{
  my $self = shift;
  my %args = ( number => "", @_ );
  my $number = $args{number};
  my $sth;
  my @info;

  if ($number !~ /^\d+$/)
  {
    $self->{error} = 1;
    $self->{errorString} = "Stock Sub Account = '$number' is invalid!<br>\n";
    return (0);
  }

  $sth = $self->{dbHandle}->read(sql => "SELECT ticker FROM stocks_tb WHERE sub_acct = ?",
                                    plug => [  $number ]);
  if ($self->{dbHandle}->didErrorOccur)
  {
    $self->{error} = 1;
    $self->{errorString} = $self->{dbHandle}->errorMessage;
    return (0);
  }
  @info = $sth->fetchrow_array;
  if (!defined $info[0])
  {
    return (-1, $info[0]);  # The stock sub account no longer exists in the stocks_tb!
  }

  my $rc = $self->{dbHandle}->write(sql => "DELETE FROM stocks_tb WHERE sub_acct = ?",
                                    plug => [ $number ]);
  if ($self->{dbHandle}->didErrorOccur)
  {
    $self->{error} = 1;
    $self->{errorString} = $self->{dbHandle}->errorMessage;
    return (0);
  }
  $self->{dbHandle}->commit;

  return (1);
}

# getStockInfo
# takes: name, ticker or number
# returns: StockObject or undef if not found
sub getStockInfo
{
  my $self = shift;
  my %args = ( @_ );
  my $type = "";
  my $var = "";
  my $sth;
  my @info;

  if (exists $args{name})
  {
    $type = "name";
    $var = $type;
    if (length $args{$type} == 0)
    {
      $self->{error} = 1;
      $self->{errorString} = "name must be specified!<br>\n";
      return undef;
    }
  }
  elsif (exists $args{number})
  {
    $type = "number";
    $var = "sub_acct";
    if (length $args{$type} == 0 || $args{$type} < 0)
    {
      $self->{error} = 1;
      $self->{errorString} = "number = '$args{$type}' is invalid!<br>\n";
      return undef;
    }
  }
  elsif (exists $args{ticker})
  {
    $type = "ticker";
    $var = $type;
    if (length $args{$type} == 0 || length $args{$type} > 10)
    {
      $self->{error} = 1;
      $self->{errorString} = "ticker = '$args{$type}' is invalid!<br>\n";
      return undef;
    }
  }
  else
  {
    $self->{error} = 1;
    $self->{errorString} = "You must specify either number, ticker or name!<br>\n";
    return undef;
  }
  my $stockObj;

  # now we select this stock from the system and return the object.
  $sth = $self->{dbHandle}->read(sql => "SELECT ticker, name, sub_acct, type, market, class FROM stocks_tb WHERE $var = ?",
                                    plug => [ $args{$type} ]);
  if ($self->{dbHandle}->didErrorOccur)
  {
    $self->{error} = 1;
    $self->{errorString} = $self->{dbHandle}->errorMessage;
    return undef;
  }
  @info = $sth->fetchrow_array;
  if (defined $info[0])
  {
    $stockObj = Portal::Accounting::Objects::StockObject->new(ticker => $info[0], name => $info[1], number => $info[2], type => $info[3],
             market => $info[4], class => $info[5], langObj => $self->{langObj});
    if ($stockObj->didErrorOccur)
    {
      $self->setError(errorString => $stockObj->errorMessage);
      return undef;
    }
  }
  else
  {
    $stockObj = undef;
  }

  return $stockObj;
}

=item @array getListOfStocks()

  required args:
  optional: tickerLike =>, orderBy => , marketOnly => 0|1
  returns: Array of stockObjects
  summary:  This routine selects stock objects.  
    If tickerLike is not specified, then it is ignored for the query.  
    If tickerLike is specified, then a ticker LIKE ? will be used.
    The orderBy string must be a valid ORDER BY possible statement for this query.
    marketOnly - defaults to 0 to select all stocks
       if 1 then stocks with a market set to NONE will be ignored

=cut
sub getListOfStocks
{
  my $self = shift;
  my %args = ( orderBy => "ticker, name", marketOnly => 0, @_ );
  my $tickerLike = (exists $args{tickerLike} ? $args{tickerLike} : "");
  my $marketOnly = 0;
  my $orderBy = $args{orderBy};
  my @stocks = ();
  my $sth;
  my @info;

  # validate input
  if (length $orderBy == 0)
  {
    $self->setError(errorString => "orderBy = '$orderBy' is invalid!<br>\n");
    return @stocks;
  }

  my $sql = "SELECT ticker, name, sub_acct, type, market, class FROM stocks_tb";
  my $where = "";
  if (length $tickerLike > 0)
  {
    $where .= " ticker LIKE '$tickerLike' ";
  }
  if ($marketOnly)
  {
    $where .= " market <> 'None' ";
  }
  if ($where ne "") {  $sql .= " WHERE " . $where;  }
  $sql .= " ORDER BY $orderBy";

  # now execute the sql statement
  $sth = $self->{dbHandle}->read(sql => $sql);
  if ($self->{dbHandle}->didErrorOccur)
  {
    $self->setError(errorString => $self->{dbHandle}->errorMessage);
    return @stocks;
  }
  while (@info = $sth->fetchrow_array)
  {
    my $stockObj = Portal::Accounting::Objects::StockObject->new(ticker => $info[0], name => $info[1],
                        number => $info[2], type => $info[3], market => $info[4], class => $info[5], langObj => $self->{langObj});
    if ($stockObj->didErrorOccur)
    {
      $self->setError(errorString => $stockObj->errorMessage);
      return @stocks;
    }
    $stocks[++$#stocks] = $stockObj;
  }

  return @stocks;
}

# getStockMarkets
# requires:
# optional:
# returns: array of markets
# summary: gathers all markets from the stock_market_tb
sub getStockMarkets
{
  my $self = shift;
  my @markets = ();
  my $sth;
  my @info;

  $sth = $self->{dbHandle}->read(sql => "SELECT name FROM stock_market_tb ORDER BY name");
  if ($self->{dbHandle}->didErrorOccur)
  {
    $self->{error} = 1;
    $self->{errorString} = $self->{dbHandle}->errorMessage;
    return @markets;
  }
  while (@info = $sth->fetchrow_array)
  {
    $markets[++$#markets] = $info[0];
  }

  return @markets;
}

# getStockClasses
# requires:
# optional:
# returns: array of classes
# summary: gathers all classes from the stock_class_tb in reverse order
sub getStockClasses
{
  my $self = shift;
  my @classes = ();
  my $sth;
  my @info;

  $sth = $self->{dbHandle}->read(sql => "SELECT name FROM stock_class_tb ORDER BY name");
  if ($self->{dbHandle}->didErrorOccur)
  {
    $self->{error} = 1;
    $self->{errorString} = $self->{dbHandle}->errorMessage;
    return @classes;
  }
  while (@info = $sth->fetchrow_array)
  {
    $classes[++$#classes] = $info[0];
  }

  return @classes;
}

=item MonitorObject getMonitorInfo(id)

    Looks up the specified monitor_tb entry.  If not found returns undef,
    else returns the Portal::Accounting::Objects::MonitorObject object.
    If an error occurs, then we set the error and return undef.

=cut
sub getMonitorInfo
{
  my $self = shift;
  my %args = (id => "", @_);
  my $id = $args{id};
  my $info = undef;
  my $sth;
  my @info;

  if ($id !~ /^(\d+)$/)
  {
    $self->setError(errorString => "id = '$id' is invalid!");
    return undef;
  }

  $sth = $self->{dbHandle}->read(sql => "SELECT xml FROM monitor_tb WHERE id = $id");
  if ($self->{dbHandle}->didErrorOccur)
  {
    $self->setError(errorString => $self->{dbHandle}->errorMessage);
    return undef;
  }
  @info = $sth->fetchrow_array;
  if (defined $info[0])
  {
    $info = Portal::Accounting::Objects::MonitorObject->new(id => $id, xml => $info[0], langObj => $self->{langObj});
    if ($info->didErrorOccur)
    {
      $self->setError(errorString => $info->errorMessage);
      return undef;
    }
  }

  return $info;
}

=item @MonitorObject getMonitorEntries(ledgerCode)

    Looks up the monitor_tb entries for the specified ledgerCode or all
    entries if ledgerCode = "".  If nothing found returns undef, else
    returns the array of Portal::Accounting::Objects::MonitorObject
    objects.  If an error occurs, then we set the error and return undef.

=cut
sub getMonitorEntries
{
  my $self = shift;
  my %args = (ledgerCode => "", @_);
  my $ledgerCode = $args{ledgerCode};
  my @entries = ();
  my $sth;
  my @info;

  my $sql = "SELECT id, ledger_code, xml FROM monitor_tb";
  if (length $ledgerCode > 0)
  {
    $sql .= " WHERE ledger_code = '$ledgerCode'";
  }
  $sql .= " ORDER BY ledger_code, id";

  $sth = $self->{dbHandle}->read(sql => $sql);
  if ($self->{dbHandle}->didErrorOccur)
  {
    $self->setError(errorString => $self->{dbHandle}->errorMessage);
    return undef;
  }
  while (@info = $sth->fetchrow_array)
  {
    my $info = Portal::Accounting::Objects::MonitorObject->new(id => $info[0], xml => $info[2], langObj => $self->{langObj});
    if ($info->didErrorOccur)
    {
      $self->setError(errorString => $info->errorMessage);
      return undef;
    }
    push @entries, $info;
  }

  return @entries;
}

=item void updateMonitorEntry(monitorObj, action)

    This routine creates or updates an entry in the monitor_tb table that
    represents a lists of accounts to monitor for the specified ledger
    that should email an alert to the specified users/email addresses when
    the specified conditions are met.  If action = "delete" then we delete
    the specified entry.

=cut
sub updateMonitorEntry
{
  my $self = shift;
  my %args = ( monitorObj => undef, action => "", @_ );
  my $monitorObj = $args{monitorObj};
  my $action = $args{action};
  my $rc;
  my @info;

  if (!defined $monitorObj)
  {
    $self->setError(errorString => "monitorObj = must be defined!<br>\n");
    return;
  }
  if (!$monitorObj->isValid())
  {
    $self->setError(errorString => $monitorObj->errorMessage);
    return;
  }
  if ($action !~ /^(|delete)$/)
  {
    $self->setError(errorString => "action = '$action' is invalid!<br>\n");
    return;
  }
  if ($action eq "delete" && $monitorObj->{id} eq "-1")
  {
    $self->setError(errorString => "You can not delete an entry with id = '-1'!<br>\n");
    return;
  }

  if ($monitorObj->{id} eq "-1")
  { # create the entry
    my $xml = $monitorObj->export();
    if ($monitorObj->didErrorOccur)
    {
      $self->setError(errorString => $monitorObj->errorMessage);
      return;
    }
    $rc = $self->{dbHandle}->write(sql => "INSERT INTO monitor_tb (ledger_code, xml) VALUES (?, ?)",
           plug => [ $monitorObj->{ledgerCode}, $xml ]);
  }
  else
  {
    if ($action eq "delete")
    { # delete the entry
      $rc = $self->{dbHandle}->write(sql => "DELETE FROM monitor_tb WHERE id = ?", plug => [ $monitorObj->{id} ]);
    }
    else
    { # update the entry
      my $xml = $monitorObj->export();
      if ($monitorObj->didErrorOccur)
      {
        $self->setError(errorString => $monitorObj->errorMessage);
        return;
      }
      $rc = $self->{dbHandle}->write(sql => "UPDATE monitor_tb SET ledger_code = ?, xml = ? WHERE id = ?",
            plug => [ $monitorObj->{ledgerCode}, $xml, $monitorObj->{id} ]);
    }
  }
  if ($self->{dbHandle}->didErrorOccur)
  {
    $self->setError(errorString => $self->{dbHandle}->errorMessage);
    return;
  }
  $self->{dbHandle}->commit;
}

=item scalar getAccountBalance(account, date, ledgerCode)

    requires: account - account number, date - date to balance to,
              ledgerCode - ledger code
    returns:  Account balance.
    date can be defined as 'now()', YYYY-/MM-/DD, MM-/DD-/YYYY

=cut
sub getAccountBalance
{
  my $self = shift;
  my %args = ( account => "", date => "", ledgerCode => "", @_);
  my $account = $args{account};
  my $date = $args{date};
  my $ledgerCode = $args{ledgerCode};
  my $sth;
  my @info;
  my $result = "0.00";  #default to a balance of 0.00

  if ($account !~ /^(\d+)$/)
  {
    $self->setError(errorString => "account = '$account' is invalid!");
    return $result;
  }
  if ($ledgerCode !~ /^(.+)$/)
  {
    $self->setError(errorString => "ledgerCode = '$ledgerCode' is invalid!");
    return $result;
  }
  if ($date !~ /^(now\(\)|\d{4}[-\/]\d{2}[-\/]\d{2}|\d{2}[-\/]\d{2}[-\/]\d{4})$/)
  {
    $self->setError(errorString => "date = '$date' is invalid!");
    return $result;
  }

  $sth = $self->{dbHandle}->read(sql => "SELECT account_balance_plus_activity($account, '$ledgerCode', '$date', 'balance') AS balance");
  if ($self->{dbHandle}->didErrorOccur)
  {
    $self->setError(errorString => $self->{dbHandle}->errorMessage);
    return $result;
  }
  @info = $sth->fetchrow_array;
  if (defined $info[0])
  {
    $result = $info[0];
  }

  return $result;
}

=item scalar getConfigEntry(name)

    requires: name - name of config_tb table entry to lookup
    returns: undef if name not found or value of entry.

=cut
sub getConfigEntry
{
  my $self = shift;
  my %args = ( name => "", @_ );
  my $name = $args{name};
  my $value = undef;
  my $sth;
  my @info;

  if ($name !~ /^(.+)$/)
  {
    $self->setError(errorString => "name = '$name' is invalid!<br>\n");
    return $value;
  }

  $sth = $self->{dbHandle}->read(sql => "SELECT value FROM config_tb WHERE name = ?", plug => [ $name ]);
  if ($self->{dbHandle}->didErrorOccur)
  {
    $self->setError(errorString => $self->{dbHandle}->errorMessage);
    return $value;
  }
  @info = $sth->fetchrow_array;
  if (defined $info[0])
  {
    $value = $info[0];
  }

  return $value;
}

=item scalar setConfigEntry(name, value)

    requires: name - name of config_tb table entry to lookup
              value - value to assign to name.
    returns: 1 if ok, 0 on error

=cut
sub setConfigEntry
{
  my $self = shift;
  my %args = ( name => "", value => "", @_ );
  my $name = $args{name};
  my $value = $args{value};
  my $sth;
  my @info;

  if ($name !~ /^(.+)$/)
  {
    $self->setError(errorString => "name = '$name' is invalid!<br>\n");
    return 0;
  }

  # see if the entry already exists.
  my $result = $self->getConfigEntry(name => $name);
  if (defined $result)
  {
    # update the entry.
    $self->{dbHandle}->write(sql => "UPDATE config_tb SET value = ? WHERE name = ?", plug => [ $value, $name ]);
  }
  else
  {
    # create the entry.
    $self->{dbHandle}->write(sql => "INSERT INTO config_tb (name, value) VALUES (?, ?)", plug => [ $name, $value ]);
  }
  if ($self->{dbHandle}->didErrorOccur)
  {
    $self->setError(errorString => $self->{dbHandle}->errorMessage);
    return 0;
  }
  $self->{dbHandle}->commit;  # make changes permanent.

  return 1;
}

=item string getCashAccountSummary( ledgerCode => , date => )

  format:  date must be ISO standard YYYY-MM-DD
  returns: HTML <table> output in the event of success;
    undef if something fails

  This routine queries the database for the list of cash/bank accounts.
  Then a table is created containing the current balance, last
  reconciled date, and last entry date for each account.

=cut
sub getCashAccountSummary
{
  my $self = shift;
  my %args = ( ledgerCode => "", date => "", @_ );
  my $ledgerCode = $args{ledgerCode};
  my $date = $args{date};
  my $sth;
  my @info;
  my $htmlString = "";

  my $phrases = $self->{langObj};
  if ($date eq "")
  {
    $date = POSIX::strftime('%Y-%m-%d', localtime);
  }
  if (length $ledgerCode == 0)
  {
    $self->setError(errorString => "ledgerCode = '$ledgerCode' is invalid!<br>\n");
    return undef;
  }

  my $sql = "";
  $sql .= "SELECT account_tb.name as name,  account_tb.number as number";
  $sql .= ", to_char(account_balance_plus_activity(account_tb.number, '$ledgerCode', '$date', 'Cash' ),'999,999,999,999,990.00') as balance";
  $sql .= ", (SELECT MAX(trans_date) from debitcredit_tb WHERE debitcredit_tb.account_num = account_tb.number AND ledger_code = '$ledgerCode') as last";
  $sql .= ", (SELECT MAX(trans_date) from debitcredit_tb WHERE debitcredit_tb.account_num = account_tb.number and debitcredit_tb.cleared = 'r' AND ledger_code = '$ledgerCode') as reconciled";
  $sql .= ", (SELECT DISTINCT(name) from ledger_tb where code = '$ledgerCode') as ledger";
  $sql .= ", account_tb.type as type";
  $sql .= " WHERE account_tb.number = ";
  $sql .= "  (SELECT DISTINCT(local_accounts_tb.acct_num) FROM local_accounts_tb";
  $sql .= "   WHERE local_accounts_tb.acct_num = account_tb.number AND account_tb.type = 0 AND ledger_code = '$ledgerCode' AND account_tb.active = 'true')";
  $sql .= " UNION SELECT account_tb.name as name,  account_tb.number as number";
  $sql .= ", to_char(sub_account_balance_plus_activity(account_tb.number, '-1', '$ledgerCode', '$date', 'Cash' ),'999,999,999,999,990.00') as balance";
  $sql .= ", (SELECT MAX(trans_date) from debitcredit_tb WHERE debitcredit_tb.account_num = account_tb.number AND ledger_code = '$ledgerCode') as last";
  $sql .= ", (SELECT MAX(trans_date) from debitcredit_tb WHERE debitcredit_tb.account_num = account_tb.number and debitcredit_tb.cleared = 'r' AND ledger_code = '$ledgerCode') as reconciled";
  $sql .= ", (SELECT DISTINCT(name) from ledger_tb where code = '$ledgerCode') as ledger";
  $sql .= ", account_tb.type as type";
  $sql .= " WHERE account_tb.number = ";
  $sql .= "  (SELECT DISTINCT(local_accounts_tb.acct_num) FROM local_accounts_tb";
  $sql .= "   WHERE local_accounts_tb.acct_num = account_tb.number AND account_tb.type = 3 AND ledger_code = '$ledgerCode' AND account_tb.active = 'true')";
  $sql .= " ORDER BY type, name";
  #return $sql;

  $sth = $self->{dbHandle}->read(sql => $sql);
  if ($self->{dbHandle}->didErrorOccur)
  {
    $self->setError($self->{dbHandle}->errorMessage);
    return undef;
  }

  $htmlString .= "<table class=\"CashSummaryTable\" border=\"0\" cellspacing=\"2\" cellpadding=\"2\">\n";
  $htmlString .= "  <tr>\n";
  $htmlString .= "    <td class=\"CashSummaryName\"><b>".$phrases->map("Account")."</b></td>\n";
  $htmlString .= "    <td class=\"spacer\">&nbsp;</td>\n";
  $htmlString .= "    <td class=\"CashSummaryBalance\"><b>".$phrases->map("Cash_Balance")."</b></td>\n";
  $htmlString .= "    <td class=\"spacer\">&nbsp;</td>\n";
  $htmlString .= "    <td class=\"CashSummaryLastEntry\"><b>".$phrases->map("Last")."</b></td>\n";
  $htmlString .= "    <td class=\"spacer\">&nbsp;</td>\n";
  $htmlString .= "    <td class=\"CashSummaryReconcile\"><b>".$phrases->map("Reconcile")."</b></td>\n";
  $htmlString .= "  </tr>\n";

  my $total = 0;
  while (my @account = $sth->fetchrow_array)
  {
    my $noISO = $date;
    my $noISOstart = "";
    $noISO =~ s/(\d{4})(-{1})(\d{2}-\d{2})/$3-$1/;
    if (not $account[3] =~ m/^(\d{2})-(\d{2})-(\d{4})$/)
    {
      $noISOstart = "OOPS! Date format is wrong ".$account[3];
    }
    else
    {
      my ($year,$month,$day) = ($3,$1,$2); #last
      my ($nY,$nM,$nD) = Today();
      ($nY,$nM,$nD) = Add_Delta_Days($nY,$nM,$nD, -31);
      if (Delta_Days($nY,$nM,$nD,$year,$month,$day) > 0)
      {
        $noISOstart = ($nM - 1) ."-".$nD."-".$nY;
      }
      else
      {
        $noISOstart = ($month - 1) ."-".$day."-".$year;
      }
    }
    $htmlString .= "  <tr>\n";
    $htmlString .= "    <td class=\"CashSummaryName\">";
    $htmlString .=        "<a href=\"/portal/js/Accounting/accounting/cgi-bin/register_display.cgi?app=Accounting&state=Registers&command=display";
    $htmlString .=         "&from_search_date=".$noISOstart."&to_search_date=$noISO&ledgerCode=$ledgerCode&ledger_db=".$account[5];
    $htmlString .=         "&account_value=".$account[0]."|".$account[1]."&account_select=&register_select_submit=";
    $htmlString .=         "\">".$account[0]."</a></td>\n";
    $htmlString .= "    <td class=\"spacer\">&nbsp;</td>\n";
#    $htmlString .= "    <td class=\"CashSummaryNumber\">".$account[1]."</td>\n";
    $htmlString .= "    <td class=\"CashSummaryBalance\">";
    if ($account[2] < 0)
    {
      $htmlString .= "<font class=\"negative\">".$account[2]."</font>";
    }
    else
    {
      $htmlString .= $account[2];
    }
    $htmlString .= "</td>\n";
    $htmlString .= "    <td class=\"spacer\">&nbsp;</td>\n";
    $htmlString .= "    <td class=\"CashSummaryLastEntry\">".$account[3]."</td>\n";
    $htmlString .= "    <td class=\"spacer\">&nbsp;</td>\n";
    $htmlString .= "    <td class=\"CashSummaryReconcile\">".$account[4]."</td>\n";
    $htmlString .= "  </tr>\n";
    $account[2] =~ s/,//;
    $total += $account[2];
  }
  $htmlString .= "  <tr>\n";
  $htmlString .= "    <td class=\"CashSummaryName\">&nbsp;</td>\n";
  $htmlString .= "    <td class=\"spacer\">&nbsp;</td>\n";
#  $htmlString .= "    <td class=\"CashSummaryNumber\">&nbsp;</td>\n";
  $htmlString .= "    <td class=\"CashSummaryBalance\"><hr>".$self->commify(sprintf('%15.2f',$total))."</td>\n";
  $htmlString .= "    <td class=\"spacer\">&nbsp;</td>\n";
  $htmlString .= "    <td class=\"CashSummaryReconcile\">&nbsp;</td>\n";
  $htmlString .= "    <td class=\"spacer\">&nbsp;</td>\n";
  $htmlString .= "    <td class=\"CashSummaryLastEntry\">&nbsp;</td>\n";
  $htmlString .= "  </tr>\n";
  $htmlString .= "</table>\n";

  return $htmlString;
}

sub commify {
  my $self = shift;
  my $text = reverse $_[0];
  $text =~ s/(\d\d\d)(?=\d)(?!\d*\.)/$1,/g;
  return scalar reverse $text;
}

=back

=cut

1;
__END__

=head1 NOTE

 All data fields are accessible by specifying the object
 and pointing to the data member to be modified on the
 left-hand side of the assignment.
 Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

Xperience, Inc. (mailto:admin@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal(3), Portal::Accounting(3), Portal::Base(3)

=cut
