# Transaction.pm - The object class that provides functions to work with transaction data in the accounting database
# Created by Jason A. Pattie, 11/07/2000.

#$Id: Transaction.pm,v 1.6 2003/08/05 16:29:47 pcxuser Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.

package Portal::Accounting::DB::Transaction;
use strict;
use Portal::Accounting::Objects::TransactionObject;
use Portal::Accounting::DB::Main;
use DBIWrapper;
use vars qw($VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Portal::Accounting::DB::Main Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '0.02';

=head1 NAME

Transaction - Object that supplies functions for interacting with transactions in the accounting database.

=head1 SYNOPSIS

  use Transaction;
  my $trans = Transaction->new(dbHandle => DBIWrapper->new(...));
  if ($trans->didErrorOccur())
  {
    die $trans->errorMessage;
  }

  # update a transaction in the database
  if (!$trans->updateTransaction($transObject))
  {
    die $trans->errorMessage;
  }

=head1 DESCRIPTION

TransactionObject is a class that stores data for an accounting
transaction.  It provides a container for a transaction that will be used
to write to the database or update an existing record in the database.

=head1 Exported FUNCTIONS

=over 4

=item TransactionObject new(dbHandle)

    Creates a new instance of the Portal::Accounting::DB::Transaction
    object.

=cut
sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  my %args = ( @_ );

  if ($self->didErrorOccur)
  {
    $self->prefixError();
    return $self;
  }

  # instantiate anything unique to this module

  # do validation
  if (!$self->isValid)
  {
    # the error is set in the isValid() method.
    return $self;
  }

  # do anything else you might need to do.

  return $self;
}

=item bool isValid(void)

 Returns 0 or 1 to indicate if the object is valid.
 The error will be available via errorMessage().

=cut

sub isValid
{
  my $self = shift;

  # make sure our Parent class is valid.
  if (not $self->SUPER::isValid())
  {
    $self->prefixError();
    return 0;
  }
  # validate our parameters.

  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->postfixError($self->genErrorString("all"));
    return 0;
  }

  return 1;
}

=item scalar insertTransaction(transactionObject => , commit =>)

    Attempts to insert the contents of the TransactionObject
    into the database.
    Returns: the reference number for the new transaction or returns -1 and sets error on failure.
    requires: TransactionObject reference
    optional: commit

=cut
sub insertTransaction
{
  my $self = shift;
  my $errorHeader = "Transaction->insertTransaction(): ";
  my %args = ( commit => 0, @_ );
  my $refNum = -1;
  my $transObj = $args{'obj'};

  if (not exists $args{'obj'})
  {
    $self->{error} = 1;
    $self->{errorString} = $errorHeader . "Object reference missing!";
    return -1;
  }
  # make sure the object passed in is really a TransactionObject
  if (ref($args{'obj'}) ne "Portal::Accounting::Objects::TransactionObject")
  {
    $self->{error} = 1;
    $self->{errorString} = $errorHeader . "Invalid TransactionObject reference (" . ref($args{'obj'}) . ")!";
    return -1;
  }
  # check the validity of the transaction object
  if ($args{'obj'}->isValid())
  {
    if ($transObj->referenceNum < 0)
    {
      # get the nextval(journal_entry_tb_ref_num_seq) of the serial sequence for the table
      my $sth = $self->{dbHandle}->read(sql => "SELECT nextval('journal_entry_tb_ref_num_seq')");
      if ($self->{dbHandle}->didErrorOccur())
      {
        $self->{error} = 1;
        $self->{errorString} = $errorHeader . "Failed to get the nextval() of the journal entry sequence!<br>\n" . $self->{dbHandle}->errorMessage;
        return -1;
      }
      my @info = $sth->fetchrow_array;
      $refNum = $info[0];
    }
    else
    {
      $refNum = $transObj->referenceNum;
    }
    # insert the TransactionObject data into the database
    $self->{dbHandle}->write(sql => "INSERT INTO journal_entry_tb (ref_num, journal_code, trans_date, payee_payer, memo, remember, ledger_code, tracking_id, orig_id) values (?, ?, ?, ?, ?, ?, ?, ?, ?)", plug => [ $refNum, $transObj->journalCode, $transObj->transDate, $transObj->payeePayer, $transObj->memo, $transObj->remember, $transObj->ledgerCode, $transObj->trackingId, $transObj->origId ]);
    if ($self->{dbHandle}->didErrorOccur())
    {
      $self->{error} = 1;
      $self->{errorString} = $errorHeader . "Failed to insert journal entry!<br>\n" . $self->{dbHandle}->errorMessage;
      $self->{dbHandle}->rollback;
      return -1;
    }
    if ($self->insertJERows(obj => $transObj, refnum => $refNum ) != 1)
    {
      $self->setError("Failed to insert row entry!<br>\n" . $self->{dbHandle}->errorMessage);
      $self->{dbHandle}->rollback;
      return -1;
    }

    if ($args{'commit'} eq "1")
    {
      # finish the transaction
      $self->{dbHandle}->commit;
    }
  }
  else
  {
    $self->{error} = 1;
    $self->{errorString} = $errorHeader . "Error!<br>\n" . $transObj->errorMessage;
    return -1;
  }

  # return the created reference number from the database
  return $refNum;
}

=item scalar updateTransaction(transObject => , commit =>)

    Attempts to update the contents of the TransactionObject
    to the database.
    requires: TransactionObject to update
    optional:  commit
    returns:  0 on failure, positive (1) value on successfully updating the reference number contained in the TransactionObject to the database

=cut
sub updateTransaction
{
  my $self = shift;
  my $errorHeader = "Transaction->updateTransaction(): ";
  my %args = ( commit => 0, @_ );
  my $refNum = -1;
  my $transObj = $args{'obj'};

  if (not exists $args{'obj'})
  {
    $self->{error} = 1;
    $self->{errorString} = $errorHeader . "Object reference missing!";
    return 0;
  }
  # make sure the object passed in is really a TransactionObject
  if (ref($args{'obj'}) ne "Portal::Accounting::Objects::TransactionObject")
  {
    $self->{error} = 1;
    $self->{errorString} = $errorHeader . "Invalid TransactionObject reference (" . ref($args{'obj'}) . ")!";
    return 0;
  }
  # check the validity of the transaction object
  if ($args{'obj'}->isValid())
  {
    $refNum = $transObj->referenceNum;
    # update the TransactionObject data in the database
    if ($self->{dbHandle}->write(sql => "UPDATE journal_entry_tb SET journal_code = ?, trans_date = ?, payee_payer = ?, memo = ?, remember = ? WHERE ref_num = ?", plug => [ $transObj->journalCode, $transObj->transDate, $transObj->payeePayer, $transObj->memo, $transObj->remember, $transObj->referenceNum ]) == 0)
    {
      # this error occurs in the case that the reference number does not exist in the database; thus, there is nothing to
      # update and the completion of the update process is not necessary or valid
      $self->{error} = 1;
      $self->{errorString} = $errorHeader . "No rows were modified!";
      return 0;
    }
    if ($self->{dbHandle}->didErrorOccur())
    {
      $self->{error} = 1;
      $self->{errorString} = $errorHeader . "Failed to update journal entry!<br>\n" . $self->{dbHandle}->errorMessage;
      $self->{dbHandle}->rollback;
      return 0;
    }
    # remove all rows from the database for the reference number
    $self->{dbHandle}->write(sql => "DELETE FROM debitcredit_tb WHERE parent_num = ?", plug => [ $refNum ]);
    if ($self->{dbHandle}->didErrorOccur())
    {
      $self->{error} = 1;
      $self->{errorString} = $errorHeader . "Failed to delete row entries!<br>\n" . $self->{dbHandle}->errorMessage;
      $self->{dbHandle}->rollback;
      return 0;
    }
    # insert the rows in the TransactionObject->TransactionRowObject(s) into the database
    if ($self->insertJERows(obj => $transObj, refnum => $refNum ) != 1)
    {
      $self->setError("Failed to insert row entry!<br>\n" . $self->{dbHandle}->errorMessage);
      $self->{dbHandle}->rollback;
      return 0;
    }

    if ($args{'commit'} eq "1")
    {
      # finish the transaction
      $self->{dbHandle}->commit;
    }
  }
  else
  {
    $self->{error} = 1;
    $self->{errorString} = $errorHeader . "Error!<br>\n" . $transObj->errorMessage;
    return 0;
  }

  return 1;
}

sub insertJERows
{
  my $self = shift;
  my %args = ( obj => undef, refnum => -1, @_ );
  my $refNum = $args{refnum};
  my $transObj = $args{'obj'};

  if ($refNum == -1)
  {
      $self->setError("Failed to insert row entry because refnum was not given!<br>");
      $self->{dbHandle}->rollback;
      return 0;
  }
  if (not defined $transObj)
  {
      $self->setError("Failed to insert row entry because 'obj' was undefined!<br>");
      $self->{dbHandle}->rollback;
      return 0;
  }
  for (my $rowcount=0; $rowcount < scalar @{ $transObj->rowObjects }; $rowcount++)
  {
    my $sql = "INSERT INTO debitcredit_tb (parent_num, trans_date, comment, sub_e_code, account_num, sub_account_num, amount, dorc, cleared, counter, ledger_code, d_amount, c_amount) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, dorcify(?, ?, debit_increases_account_num(?)), dorcify(?, ?, class_action_account_num(?, 'cash')) )";
    my $r = $transObj->{rowObjects}->[$rowcount];

    $self->{dbHandle}->write(sql => $sql, plug => [ $refNum, $r->rowDate, $r->comment, $r->subEntity, $r->accountNum, $r->subAccountNum, $r->amount, $r->dorc, $r->cleared, $rowcount, $transObj->ledgerCode, $r->amount, $r->dorc, $r->accountNum, $r->amount, $r->dorc, $r->accountNum ]);
    if ($self->{dbHandle}->didErrorOccur())
    {
      $self->setError("Failed to insert row entry!<br>\n" . $self->{dbHandle}->errorMessage);
      return 0;
    }
  }
  return 1;
}

=item ref getTransaction(referenceNum)

    Retrieves the data from a transaction row in the database
    for the given reference number.  Returns a
    TransactionObject containing the data.

=cut
sub getTransaction
{
  my $self = shift;
  my $errorHeader = "Transaction->getTransaction(): ";
  my %args = ( @_ );
  my $refNum = $args{'refNum'};
  my $transObj;

  if (not exists $args{'refNum'})
  {
    $self->{error} = 1;
    $self->{errorString} = $errorHeader . "Reference number missing!";
    return;
  }

  # retrieve the data for the reference number from the database and place it in the TransactionObject
  my @info = $self->{dbHandle}->read(sql => "SELECT journal_code, trans_date, payee_payer, memo, remember, ledger_code, tracking_id, orig_id FROM journal_entry_tb WHERE ref_num = ?", plug => [ $refNum ])->fetchrow_array;
  my $journalCode = $info[0];
  my $transDate = $info[1];
  my $payeePayer = $info[2];
  my $memo = $info[3];
  my $remember = $info[4];
  my $ledgerCode = $info[5];
  my $trackingId = $info[6];
  my $origId = $info[7];
  my $sth = $self->{dbHandle}->read(sql => "SELECT trans_date, comment, sub_e_code, account_num, sub_account_num, amount, dorc, cleared, d_amount, c_amount FROM debitcredit_tb WHERE parent_num = ?", plug => [ $refNum ]);
  my $rowcount = 0;
  my @rowObjectArray = [];
  while (@info = $sth->fetchrow_array)
  {
    $rowObjectArray[$rowcount] = Portal::Accounting::Objects::TransactionRowObject->new(rowDate => "$info[0]", comment => "$info[1]", subEntity => "$info[2]", accountNum => $info[3], subAccountNum => $info[4], amount => "$info[5]", cleared => "$info[6]", d_amount => "$info[7]", c_amount => "$info[8]" );
    $rowcount++;
  }
  $transObj = Portal::Accounting::Objects::TransactionObject->new(referenceNum => $refNum, ledgerCode => "$ledgerCode", journalCode => "$journalCode", transDate => "$transDate", payeePayer => "$payeePayer", memo => "$memo", remember => "$remember", rowObjects => \@rowObjectArray, trackingId => $trackingId, origId => $origId);

  return $transObj;
}

=item scalar deleteTransaction(referenceNum => , commit =>)

    Deletes the transaction from the database with the given
    reference number.
    requires: referenceNum
    optional:  commit
    returns:  1 on sucess, 0 on failure

=cut
sub deleteTransaction
{
  my $self = shift;
  my $errorHeader = "Transaction->deleteTransaction(): ";
  my %args = ( commit => 0, @_ );
  my $refNum = $args{'refNum'};

  if (not exists $args{'refNum'})
  {
    $self->{error} = 1;
    $self->{errorString} = $errorHeader . "Reference number missing!";
    return 0;
  }

  # delete the desired reference number from the database transaction tables
  my $return1 = $self->{dbHandle}->write(sql => "DELETE FROM journal_entry_tb WHERE ref_num = ?", plug => [ $refNum ]);
  if ($self->{dbHandle}->didErrorOccur())
  {
    $self->{error} = 1;
    $self->{errorString} = $errorHeader . "Failed to delete journal entry!<br>\n" . $self->{dbHandle}->errorMessage;
    $self->{dbHandle}->rollback;
    return 0;
  }
  my $return2 = $self->{dbHandle}->write(sql => "DELETE FROM debitcredit_tb WHERE parent_num = ?", plug => [ $refNum ]);
  if ($self->{dbHandle}->didErrorOccur())
  {
    $self->{error} = 1;
    $self->{errorString} .= $errorHeader . "Failed to delete row entries!<br>\n" . $self->{dbHandle}->errorMessage;
    $self->{dbHandle}->rollback;
    return 0;
  }
  if ($args{'commit'} eq "1")
  {
    # finish the transaction
    $self->{dbHandle}->commit;
    if ($self->{dbHandle}->didErrorOccur())
    {
      $self->{error} = 1;
      $self->{errorString} .= $errorHeader . "Error occurred committing work!<br>\n" . $self->{dbHandle}->errorMessage;
      return 0;
    }
  }

  if ($return1 == 0 or $return2 == 0)
  {
    if ($return1 == 0 and $return2 == 0)
    {
      $self->{errorString} .= $errorHeader . "No records affected in either journal_entry_tb or debitcredit_tb tables!";
      return 2;
    }
    elsif ($return1 == 0)
    {
      $self->{errorString} .= $errorHeader . "No records affected in the journal_entry_tb table!";
      return 3;
    }
    elsif ($return2 == 0)
    {
      $self->{errorString} .= $errorHeader . "No records affected in the debitcredit_tb table!";
      return 4;
    }
  }

  return 1;
}

1;
__END__

=head1 AUTHOR

Xperience, Inc. (mailto:xiwa@pcxperience.com)

=head1 SEE ALSO

perl(1), DBIWrapper(3), TransactionObject(3), TransactionRowObject(3)

=cut
