# StockQuote.pm - provides an object that will use "wget" to retrieve stock quote data (delayed and historical)
# from the Yahoo! Finance site.
# Author: Jason A. Pattie
#   Date: 03/02/2001
#$Id: StockQuote.pm,v 1.3.2.5 2005/07/06 14:31:47 moreejt Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.

package Portal::Accounting::Data::StockQuote;

use strict;
use vars qw($VERSION @ISA @EXPORT);
use POSIX qw(tmpnam);
require Exporter;

@ISA = qw(Exporter);
@EXPORT = qw(getStockQuotes getHistoricalQuotes);

$VERSION = '0.01';

#     Name: getStockQuotes
#  Summary: Use http://finance.yahoo.com/ site to retrieve a comma-separated-values file with the data for the stock
#           symbols in the format "<ticker>",<last price>,"<date M[M]/D[D]/YYYY>".  If the data or the symbol does not
#           exist, "N/A" is returned in the <date> field.
# Requires: tickerList (a reference to a list of ticker symbols for stocks to look up the last price)
#  Returns: array with the first element containing an error message (if any) and the second element containing a
#           multi-dimensional hash with {<ticker>}{<date>}-><last price> on sucess or undef on error
sub getStockQuotes
{
  my %args = ( tickerList => undef, @_ );
  my $errorString = "StockQuote::getStockQuotes() - Error: ";
  my @tickerList = @{$args{'tickerList'}};
  my %stockQuotes = ();

  if ((scalar @tickerList) == 0)
  {
    return ($errorString . "tickerList empty.  tickerList must be a list of tickers.", undef);
  }
  # encode the list of stocks to retrieve using '+' as the separator
  my $stockList = join("+", @tickerList);
  # build the URL for retrieving the stock quote information file
  my $URL = "\"http://finance.yahoo.com/d/quotes.csv?s=$stockList&f=sl1d1&e=.csv\"";
  # create a random file in /tmp/...
  my $tempFile = tmpnam();
  # make sure that "wget" exists on the system and is in the right place
  if (not -e "/usr/bin/wget")
  {
    return ($errorString . "/usr/bin/wget does not exist!  Make sure the 'wget' package is installed and in the necessary location.", undef);
  }
  # use "wget" to retrieve the URL
  my $call = "/usr/bin/wget -q -O $tempFile $URL > /dev/null 2>&1";
  my @args = ($call);
  my $result = system(@args);
  if ($result != 0)
  {
    unlink $tempFile;
    return ($errorString . "System call to retrieve stock prices failed. '$call'", undef);
  }
  # open the temporary file which should contain the data for the stocks
  if (!open(STOCKQUOTES, "<$tempFile"))
  {
    return ($errorString . "Could not open temporary file.", undef);
  }
  # read each line of data, splitting on commas
  while (my $line = <STOCKQUOTES>)
  {
    (my $symbol, my $price, my $date) = split(',', $line, 3);
    chomp $date;
    if ($date !~ /N\/A/) # valid value returned for this row of data in the file
    {
      $symbol =~ s/\"//g; # remove double quotes from symbol and date information
      $date =~ s/\"//g;
      
      (my $stockTicker) = grep /^$symbol$/i, @tickerList;  # we should probably check here to make sure Yahoo! is not returning to us stock tickers/quotes we didn't ask for
      $stockQuotes{$stockTicker} = ();
      $stockQuotes{$stockTicker}{$date} = $price;
    }
  }
  # close the temporary file
  close(STOCKQUOTES);

  # remove the temporary file
  unlink $tempFile;

#  if (scalar keys %stockQuotes == 0)
#  {
#    return ($errorString . "No stock quotes returned.", undef);
#  }

  return ("", \%stockQuotes);
}

#     Name: getHistoricalQuotes
#  Summary: Use http://ichart.yahoo.com/ site to retrieve a comma-separated-values file with the data for the stock
#           symbol in the format <date D[D]-MMM-YY>,<open>,<high>,<low>,<close>,<volume>.
#           Date format changes for monthly gatherings.  The close price exists for the last day of the month.
#           <MMM YY>
#           If the data or the symbol does not exist, "No data available for <ticker>" is returned in the first line
#           of the file.
# Requires: ticker (the ticker symbol to lookup close price for the date range)
#           startMonth (the beginning of the date range to lookup closing price information)
#           startDay
#           startYear
#           endMonth (the end of the date range to lookup closing price information)
#           endDay
#           endYear
# Optional: interval (d = daily, w = weekly, m = monthly; defaults to daily in the absence of the argument or in the
#                     case of an unrecognized value)
#  Returns: array with the first element containing an error message (if any) and the second element containing a
#           hash with {<date>}-><last price> on sucess or undef on error
sub getHistoricalQuotes
{
  my %args = ( ticker => undef, startMonth => undef, startDay => undef, startYear => undef,
               endMonth => undef, endDay => undef, endYear => undef, interval => undef, @_ );
  my $errorString = "StockQuote::getHistoricalQuotes() - Error: ";
  my $symbol = $args{'ticker'};
  my $startMonth = $args{'startMonth'};
  my $startDay = $args{'startDay'};
  my $startYear = $args{'startYear'};
  my $endMonth = $args{'endMonth'};
  my $endDay = $args{'endDay'};
  my $endYear = $args{'endYear'};
  my $interval = $args{'interval'};
  my %historicalQuotes = ();

  if ((not defined $symbol) || $symbol eq "")
  {
    return ($errorString . "ticker empty or not specified.", undef);
  }
  if ((not defined $startMonth) || $startMonth eq "")
  {
    return ($errorString . "startMonth empty or not specified.", undef);
  }
  if ($startMonth !~ /^((0?[1-9])|(1[0-2]))$/)
  {
    # invalid month value
    return ($errorString . "Invalid value for startMonth (" . $startMonth . ").", undef);
  }
  $startMonth--;  # fixup per Yahoo's change
  if ((not defined $startDay) || $startDay eq "")
  {
    return ($errorString . "startDay empty or not specified.", undef);
  }
  if ($startDay !~ /^((0?[1-9])|([12][0-9])|(3[01]))$/)
  {
    # invalid day value
    return ($errorString . "Invalid value for startDay (" . $startDay . ").", undef);
  }
  if ((not defined $startYear) || $startYear eq "")
  {
    return ($errorString . "startYear empty or not specified.", undef);
  }
  if ($startYear !~ /^[0-9]+$/)
  {
    # invalid year value
    return ($errorString . "Invalid value for startYear (" . $startYear . ").", undef);
  }
  if ((not defined $endMonth) || $endMonth eq "")
  {
    return ($errorString . "endMonth empty or not specified.", undef);
  }
  if ($endMonth !~ /^((0?[1-9])|(1[0-2]))$/)
  {
    # invalid month value
    return ($errorString . "Invalid value for endMonth (" . $endMonth . ").", undef);
  }
  $endMonth--;  # fixup per Yahoo's change
  if ((not defined $endDay) || $endDay eq "")
  {
    return ($errorString . "endDay empty or not specified.", undef);
  }
  if ($endDay !~ /^((0?[1-9])|([12][0-9])|(3[01]))$/)
  {
    # invalid day value
    return ($errorString . "Invalid value for endDay (" . $endDay . ").", undef);
  }
  if ((not defined $endYear) || $endYear eq "")
  {
    return ($errorString . "endYear empty or not specified.", undef);
  }
  if ($endYear !~ /^[0-9]+$/)
  {
    # invalid year value
    return ($errorString . "Invalid value for endYear (" . $endYear . ").", undef);
  }
  if ((not defined $interval) || ($interval ne "d" && $interval ne "w" && $interval ne "m"))
  {
    $interval = "d";
  }

  # build the URL for retrieving the stock quote information file
  my $URL = "\"http://ichart.finance.yahoo.com/table.csv?s=$symbol&a=$startMonth&b=$startDay&c=$startYear&d=$endMonth&e=$endDay&f=$endYear&g=$interval&ignore=.csv\"";
  # create a random file in /tmp/...
  my $tempFile = tmpnam();
  # make sure that "wget" exists on the system and is in the right place
  if (not -e "/usr/bin/wget")
  {
    return ($errorString . "/usr/bin/wget does not exist!  Make sure the 'wget' package is installed and in the necessary location.", undef);
  }
  # use "wget" to retrieve the URL
  my $call = "/usr/bin/wget -q -O $tempFile $URL > /dev/null 2>&1";
  my @args = ($call);
  my $result = system(@args);
  if ($result != 0)
  {
    unlink $tempFile;
    return ($errorString . "System call to retrieve stock prices failed. $call", undef);
  }
  # open the temporary file which should contain the data for the stocks
  if (!open(HISTORICALQUOTES, "<$tempFile"))
  {
    return ($errorString . "Could not open temporary file.", undef);
  }
  # read the first line to determine if there is valid data in the file and in what order it should be processed
  my $line = <HISTORICALQUOTES>;
  my @fields = split(',', $line);
  if ((scalar @fields) <= 1)
  {
    # not enough fields returned
    return ($errorString . "Invalid data returned. (Not enough fields)", undef);
  }
  my %fieldOrder = ();
  for (my $labelCount=0; $labelCount < (scalar @fields); $labelCount++)
  {
    if ($fields[$labelCount] =~ /date|week of/i)
    {
      $fieldOrder{'DATE'} = $labelCount;
    }
    elsif ($fields[$labelCount] =~ /^Close$/)
    {
      $fieldOrder{'CLOSE'} = $labelCount;
    }
  }
  # read each line of data, splitting on commas
  while ($line = <HISTORICALQUOTES>)
  {
    my @fieldValues = split(',', $line);
    my $date = $fieldValues[$fieldOrder{'DATE'}];
    my $price = $fieldValues[$fieldOrder{'CLOSE'}];

    my $day;
    my $month;
    my $year;
    # massage date format into DD-MMM-YYYY for recognized date formats
    if ($date =~ /^(\w+) (\d+)$/)
    {
      $month = $1;
      $year = $2;

      if ($year =~ /\d{2}/) # if two-digit year, convert it to four-digit year; otherwise, leave it alone
      {
        if (($year < 100) && ($year >= 50))
        {
          $year += 1900; # change year format into 1900 + year using 1950 as the starting point; this code will only work up till the year 2049 for two-digit years
        }
      }

      if ($month =~ /apr/i || $month =~ /jun/i || $month =~ /sep/i || $month =~ /nov/i)
      {
        $day = "30";
      }
      elsif ($month =~ /jan/i || $month =~ /mar/i || $month =~ /may/i || $month =~ /jul/i || $month =~ /aug/i || $month =~ /oct/i || $month =~ /dec/i)
      {
        $day = "31";
      }
      elsif ($month =~ /feb/i)
      {
        # use the date command to determine the ending day of february for the given year
        $day = `/bin/date -d "01-Mar-$year 1 day ago" +%d`;
        chomp $day;
      }
    }
    elsif ($date =~ /^(\d+)-(\w+)-(\d+)$/)
    {
      $day = $1;
      $month = $2;
      $year = $3;

      if ($year =~ /\d{2}/) # if two-digit year, convert it to four-digit year; otherwise, leave it alone
      {
        if (($year < 100) && ($year >= 50))
        {
          $year += 1900; # change year format into 1900 + year using 1950 as the starting point; this code will only work up till the year 2049 for two-digit years
        }
      }
    }
    $date = $day . "-" . $month . "-" . $year;
    $historicalQuotes{$date} = $price;
  }
  # close the temporary file
  close(HISTORICALQUOTES);

  # remove the temporary file
  unlink $tempFile;

#  if (scalar keys %historicalQuotes == 0)
#  {
#    return ($errorString . "No stock quotes returned.", undef);
#  }

  return ("", \%historicalQuotes);
}

1;
__END__

=head1 NAME

StockQuote - Package containing methods to retrieve stock quote data (delayed and historical) from the Internet.

=head1 SYNOPSIS

  use Portal::Accounting::Data::StockQuote;
  my @list = ("RHAT", "IBM", "LNUX");
  my @stockQuotes = Portal::Accounting::Data::StockQuote->getStockQuotes(tickerList => \@list);
  if ($stockQuotes[0] ne "")
  {
    die $stockQuotes[0];
  }

=head1 DESCRIPTION

The StockQuote package exports two functions that allow retrieval of financial
stock quote data from the Internet.  One function retrieves current (delayed)
stock quotes for a given list of stock tickers.  The other function retrives
historical stock quotes for a single stock ticker over a specified
period of time.

=head1 Exported FUNCTIONS

  list getStockQuotes(tickerList)
    Takes a reference to a list of ticker symbols and retrieves a
    current (delayed) quote for each ticker symbol in the tickerList
    list.

  list getHistoricalQuotes(ticker, startMonth, startDay, startYear,
                           endMonth, endDay, endYear, interval)
    Takes a single ticker symbol along with the period of time broken
    down into the individual components of month, day, and year over
    which to retrieve historical data.  The interval specifies the
    frequency with which the data is to be retrieved (i.e.,
    monthly = 'm', weekly = 'w', or daily = 'd').

=head1 AUTHOR

PC & Web Xperience, Inc. (mailto:admin@pcxperience.com)

=head1 SEE ALSO

perl(1)

=cut
