# Ledger.pm
# Created by James Pattie & Jason A. Pattie, 11/07/2000.
#$Id: Ledger.pm,v 1.47 2004/02/11 21:06:45 moreejt Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.

=head1 NAME

Ledger - Object used to output Ledger screens

=head1 SYNOPSIS

  use Portal::Accounting::Ledger;
  my $obj = Portal::Accounting::Ledger->new;
  if ($obj->didErrorOccur())
  {
    die $obj->errorMessage();
  }
  $obj->run(command => $command);

=head1 DESCRIPTION

Ledger is an accounting ledger class.

=cut
package Portal::Accounting::Ledger;
use strict;
use Portal::Accounting::DB::Main;
use Portal::Accounting::Base;
use Portal::Accounting::DB::Ledger;
use Portal::Accounting::Data::Config;
use Portal::Accounting::Helpers::WatchList;
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Portal::Accounting::Base Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '0.03';

# global variables.

# modify to represent the commands this state provides.
my %commands = (
              display => "Ledger Summary",            #always checks for output variable
              closeLedger => "Close Ledger",
              ledgerFrame => "",
              ledgerMenu  => "Leger Menu",
              closeOut => "Close books",
              reOpen => "Open closed books",
              editLedger => "Edit Ledger Properties", #same as create ledger, submits to saveLedger
              createLedger => "Create New Ledger",    #same as editLedger, submits to saveCreateLedger
              saveCreateLedger => "Create New Ledger",   #saves settings and submits directly to display
              saveEditLedger => "Save Ledger Settings",   #saves settings and submits directly to display
              confirmDelete => "Confirm Delete Ledger",
              deleteLedger => "Delete Ledger",
              watchList => "Watch List"
              );

=head1 Exported FUNCTIONS


B<NOTE>: I<bool> = 1(true), 0(false)

=over 4

=item scalar new(langObj)

 Creates a new instance of the Portal::Accounting::Ledger module.
 See Portal::Accounting::Base(3) for a listing of required arguments.

=cut

sub new
{
  my $class = shift;
  my $self;
  eval { $self = $class->SUPER::new(@_); };
  if ($@)
  {
    die $@;
    return $self;
  }
  my %args = ( @_ );

  if ($self->didErrorOccur)
  {
    $self->prefixError();
    return $self;
  }

  # do validation
  if (!$self->isValid)
  {
    # the error is set in the isValid() method.
    return $self;
  }

  # do anything else you might need to do.
  $self->{commands} = \%commands;

  return $self;
}

=head2 bool isValid(void)

    Returns 0 or 1 to indicate if the object is valid.  The error will be
    available via errorMessage().

=cut
sub isValid
{
  my $self = shift;

  # make sure our Parent class is valid.
  if (!$self->SUPER::isValid())
  {
    $self->prefixError();
    return 0;
  }

  # do validation code here.

  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->postfixError($self->genErrorString("all"));
    return 0;
  }

  return 1;
}

=head2 HTMLObject run(command)

    Runs the specified command and returns the resulting HTMLObject.

=cut
sub run
{
  my $self = shift;
  my %args = (command => "", @_ );
  my $command = $args{command};
  my $doc;
  my $siteDir = $self->{portalConfigObj}->{siteDir};
  my $phrases = $self->{langObj};

  if (!exists $commands{$command})
  {
    $self->setError(sprintf($phrases->map("commandNotKnown"), $command));
    return undef;
  }

  if ($command eq "ledgerMenu")
  {
    $doc = $self->ledgerMenu();
  }
  elsif ($command eq "watchList")
  {
    $doc = $self->watchList();
  }
  elsif ($command eq "ledgerFrame")
  {
    $doc = $self->ledgerFrame();
  }
  elsif ($command eq "logout")
  {
    $doc = $self->logout;
  }
  elsif ($command eq "createLedger" || $command eq "editLedger")
  {
    $doc = $self->editLedger;
  }
  elsif ($command eq "saveEditLedger" || $command eq "saveCreateLedger")
  {
    $doc = $self->saveLedger;
  }
  elsif ($command eq "closeLedger")
  {
    $doc = $self->closeLedger;
  }
  elsif ($command eq "deleteLedger")
  {
    $doc = $self->deleteLedger;
  }

  if ($command eq "display")
  {
    $doc = $self->display;
  }

  if (not defined $doc)
  {
    if ($self->didErrorOccur)
    {
      return undef;
    }
    else
    {
      $self->setError($self->errorMessage . "Command: $command returned undefined document!<br>\n");
      return undef;
    }
  }
  return $doc;
}

sub display
{
  my $self = shift;
  my %args = ( command => "", @_ );
  my $command = 'display';
  my $errStr = "Portal::Accounting::Ledger->display()  - Error!<br>\n";
  my $siteDir = $self->{oldCGIDir};
  my $portalCGIDir = $self->{portalCGIDir};
  my $phrases = $self->{langObj};
  my $currVersion  = $self->{release} . "." . $self->{DBVersion};
  my $mainHeight = "600";
  my $mainWidth = "1024";

  my $url = $self->{methods}->createBaseURL(type => "App", linkType => "cgi", appConfigObj => $self->{sessionObj}->{store}->{companyAppObj});

  my $mainDB = Portal::Accounting::DB::Main->new(dbHandle => $self->{dbHandle}, langObj => $phrases);
  if ($mainDB->didErrorOccur)
  {
    $self->prefixError("Could not initialize DB::Main object!<br>".$mainDB->errorMessage."\n");
    return undef;
  }
  my $doc = HTMLObject::Normal->new;
  if (not defined $doc || $doc->didErrorOccur)
  {
    $self->setError(errorString => "Could not create HTMLObject<br>\n");
    return undef;
  }

  my $ledger = $self->{input}->{ledger};
  my $ledgerCode = $self->{input}->{ledgerCode};
  my $color = $self->{sessionObj}->{store}->{ledgers}->{$ledger}->{color};

  my $portfolioPhrase = $phrases->map("Portfolio");

  # generate javascript functions needed.
  $self->{methods}->includeCoreJSFiles(doc => $doc);
  if ($self->{methods}->didErrorOccur)
  {
    $self->setError($self->{methods}->errorMessage);
    return undef;
  }

  $doc->setTitle($ledger." ".$phrases->map("Summary"));
  $doc->setFocus("body");
  $doc->print("<center><h2><font color=\"$color\">$ledger</font> ".$phrases->map("Ledger_Summary")."</h2></center>");

  $doc->print(<<"ENDOF");
  <table border="0" width="100%">
    <tr>
      <td width="15%" valign="top">
ENDOF
  $doc->print("        <a href=\"".$siteDir."/close_fiscal_year.cgi?ledger=$ledger&ledgerCode=$ledgerCode\">".$phrases->map("Close_Books")."</a><br>");
  $doc->print("        <a href=\"".$siteDir."/reopen_fiscal_year.cgi?ledger=$ledger&ledgerCode=$ledgerCode\">".$phrases->map("Open_Books")."</a><br>");
  $doc->print("        <a href=\"".$siteDir."/initial_balance_sheet.cgi?ledger=$ledger&ledgerCode=$ledgerCode\">".$phrases->map("Initial_Account_Balances")."</a>");
  $doc->print("        <hr width=\"50%\" align=\"left\">");
  $doc->print("        <a href=\"".$siteDir."/sub_entities.cgi?ledger=$ledger&ledgerCode=$ledgerCode\">".$phrases->map("Sub_Entities")."</a><br>");
  $doc->print("        <a href=\"\" onclick=\"javascript: { windowName = getTimeStamp(); window.open('$url?app=Accounting&state=Ledger&windowName='+windowName+'&command=editLedger&ledgerCode=$ledgerCode&ledger=$ledger&ledgerOpen=1', windowName , 'width=$mainWidth,height=$mainHeight,scrollbars,status,toolbar'); return false; }\">".$phrases->map("Ledger")." ".$phrases->map("Options")."</a>");
  $doc->print(<<"ENDOF");
        <hr width="50%" align="left">
        <a href="$siteDir/prg_config.cgi?ledger=$ledger&ledgerCode=$ledgerCode">Old Style Configuration</a>
        <hr width="50%" align="left">
ENDOF
  $doc->print("        <a href=\"$siteDir/federal_deposit.cgi?ledger=$ledger&ledgerCode=$ledgerCode\">".$phrases->map("Fed_Deposit")."</a>");
  $doc->print(<<"ENDOF");
      </td>
      <td valign="top">
        <center>
        <style type="text/css">
        <!--
          .CashSummaryTitle { font-weight: bold; text-decoration: none; }
          .CashSummaryTable { }
          .CashSummaryName { text-transform: capitalize; }
          .CashSummaryBalance {text-align: right; }
          .CashSummaryReconcile { }
          .CashSummaryLastEntry { }
          .negative { color: #ff0000; }
        -->
        </style>
ENDOF
  my $html = $mainDB->getCashAccountSummary(ledgerCode => $self->{input}->{ledgerCode});
  if ($mainDB->didErrorOccur)
  {
    $html = $mainDB->errorMessage;
  }
  if (not defined $html)
  {
    $html = "Error during Cash account Summary!<br>" . $mainDB->errorMessage();
  }
  $doc->print($html);
  $doc->print(<<"ENDOF");
      </td>
    </tr>
  </table>
  <hr noshade size="2">
  <iframe name="watchList" align="top" frameborder="0" width="100%" scrolling="yes" Height="600" src="?app=Accounting&state=Ledger&command=watchList&ledger=$ledger&ledgerCode=$ledgerCode"></iframe>
ENDOF
  return $doc;
}

sub watchList
{
  my $self = shift;
  my $ledger = $self->{input}->{ledger};
  my $ledgerCode = $self->{input}->{ledgerCode};
  my $phrases = $self->{langObj};
  my $doc = HTMLObject::Normal->new;
  if (not defined $doc || $doc->didErrorOccur)
  {
    $self->setError(errorString => "Could not create HTMLObject<br>\n");
    return undef;
  }
  # instantiate a WatchList Helper object
  my $watchListHelper = Portal::Accounting::Helpers::WatchList->new(dbHandle => $self->{dbHandle}, configObj => $self->{configObj}, langObj => $self->{langObj});
  if ($watchListHelper->didErrorOccur)
  {
    $doc->print("Could not create WatchList Helper object<br>\n". $watchListHelper->errorMessage);
    return $doc;
  }

  $doc->setTitle($phrases->map("WatchList").":".$ledger);
  $doc->setFocus("body");
  $doc->print(<<"ENDOF");  # output the CSS definitions for the Stock Watch List
  <center>
      <style type="text/css">
      <!--
        .WLtitle { font-weight: bold; text-decoration: none; }
        .WLgroupName { background-color: #ffffff; font-weight: bold; font-size: x-large; text-decoration: none; }
        .WLoddGroup { background-color: #ffFFff; }
        .WLevenGroup { background-color: #fFEFfF; }
        .WLquoteTable {}
        .WLcompanyNameLabel {}
        .WLtickerSymbolLabel {}
        .WLtotalSharesLabel {}
        .WLaverageCostPerShareLabel {}
        .WLlastPriceLabel {}
        .WLaverageCostPerShareLabel {}
        .WLnetChangeLabel {}
        .WLpercentChangeLabel {}
        .WLvolumeTradedLabel {}
        .WLtotalMarketValueLabel {}
        .WLgainLossLabel {}
        .WLcompanyName { text-transform: capitalize; }
        .WLtickerSymbol {}
        .WLtotalShares { text-align: right; }
        .WLcost { text-align: right; }
        .WLtotalCost { text-align: right; font-weight: bold; }
        .WLaverageCostPerShare { text-align: right; }
        .WLlastPrice { font-weight: bold; text-align: right; }
        .WLnetIncrease { color: #00AA00; text-align: right; }
        .WLnetDecrease { color: #FF0000; text-align: right; }
        .WLnetChange { text-align: right; }
        .WLnetIncreaseImage {}
        .WLnetDecreaseImage {}
        .WLpercentIncrease { color: #00AA00; text-align: right; }
        .WLpercentDecrease { color: #FF0000; text-align: right; }
        .WLpercentChange { text-align: right; }
        .WLvolumeTraded { text-align: right; }
        .WLmarketValue { text-align: right; }
        .WLtotalMarketValue { text-align: right; font-weight: bold; }
        .WLgain { color: #00AA00; text-align: right; }
        .WLloss { color: #FF0000; text-align: right; }
        .WLnoGainLossChange { text-align: right; }
        .WLgainTotal { color: #00AA00; text-align: right; font-weight: bold; }
        .WLlossTotal { color: #FF0000; text-align: right; font-weight: bold; }
        .WLnoGainLossChangeTotal { text-align: right; font-weight: bold; }
        .WLquoteDate {}
      -->
      </style>
ENDOF
  my $stockHTML = $watchListHelper->getWatchList(ledgerCode => $ledgerCode, ledger => $ledger);
  if ($watchListHelper->didErrorOccur)
  {
    $doc->print("<b><i>" . $phrases->map("errorLookingUpStockWatchListInfo") . "</i></b><br>\n" . $watchListHelper->errorMessage);
  }
  else
  {
    $doc->print($stockHTML);
  }
  $doc->print("<a href=\"index.cgi?app=Accounting&state=Investments%3A%3AWatchList&command=display&ledgerCode=$ledgerCode&ledger=$ledger\" title=\"".$phrases->map("Edit_List")."\" alt=\"".$phrases->map("Edit_List")."\">");
  $doc->print("<span class=\"WLtitle\">".$phrases->map("Edit WatchLists")."</span>");
  $doc->print("</a>");
  $doc->print(<<"ENDOF");
         </center>
ENDOF

  return $doc;
}

sub closeLedger
{
  my $self = shift;
  my $doc;
  my $errStr = "Portal::Accounting::Ledger->closeLedger()  - Error!<br>\n";
  my $siteDir = $self->{portalConfigObj}->{siteDir};
  my $phrases = $self->{langObj};
  my $windowName = $self->{input}->{windowName};

  $doc = HTMLObject::Normal->new;
  if (not defined $doc || $doc->didErrorOccur)
  {
    $self->setError( $self->errorMessage . "Could not create HTMLObject<br>\n");
    return undef;
  }

  # remove this ledger from the window list.
  if (!exists $self->{sessionObj}->{store}->{windows}->{$windowName})
  {
    $self->setError(errorString => $self->errorMessage . $errStr . "Window = '$windowName' does not exist!<br>\n");
    return undef;
  }
  delete $self->{sessionObj}->{store}->{windows}->{$windowName};
  $self->{sessionObj}->{store}->{changed} = 1;

  # generate the document to close this window.
  my $onLoadCode = "window.close();\n";
  $doc->setOnload(code => $onLoadCode);
  $doc->setTitle($phrases->map("Close_Window")." - $windowName");
  $doc->setFocus("body");
  $doc->print($phrases->map("Close_Window")." - $windowName\n");

  return $doc;
}

sub ledgerMenu
{
  my $self = shift;
  my %args = (ledger => "", ledgerCode => "", @_ );
  my $doc;
  my $errStr = "Portal::Accounting::Main->ledgerMenu()  - Error!<br>\n";
  my $phrases = $self->{langObj};
  my $ledger = $self->{input}->{ledger};
  my $ledgerCode = $self->{input}->{ledgerCode};
  my $target = "";
  my $windowName;
  my $mainHeight = "600";
  my $mainWidth = "1024";

  my $url = $self->{methods}->createBaseURL(type => "App", linkType => "cgi", appConfigObj => $self->{sessionObj}->{store}->{companyAppObj});
  my $jsUrl = $self->{methods}->createBaseURL(type => "App", linkType => "js", appConfigObj => $self->{sessionObj}->{store}->{companyAppObj}) . "accounting/cgi-bin";

  if (exists $self->{input}->{windowName})
  {  $windowName = $self->{input}->{windowName}; }
  if ($windowName eq "")
  {
    $self->setError( errorString => "Did not receive a windowName for parent of frameset<br>\n");
    return undef;
  }
  $target = $windowName . "Data";
  if ($ledgerCode eq "")
  {
    $self->setError( errorString => "No ledger code specified<br>\n");
    return undef;
  }
  if ($ledger eq "")
  {
    $self->setError( errorString => "No ledger specified<br>\n");
    return undef;
  }
  $doc = HTMLObject::Normal->new();
  if (not defined $doc || $doc->didErrorOccur)
  {
    $self->setError( errorString => "Could not create HTMLObject<br>\n");
    return undef;
  }

  $doc->setFocus("javascript");
  $doc->print(<<"END_OF_CODE");
function getTimeStamp()
{
  var timeStamp = new Date();
  return timeStamp.getTime();
}
END_OF_CODE

  $doc->setBase(target => $target);
  $doc->setFocus("body");
  $doc->print(<<"END_OF_CODE");
<table width="100%">
  <tr>
    <td align="left">
END_OF_CODE
  $doc->print("\n | <a href=\"$url?app=Accounting&state=Ledger&command=display&ledger=$ledger&ledgerCode=$ledgerCode\">".$phrases->map("Summary")."</a>");
  $doc->print("\n | <a href=\"$jsUrl/journal_activity.cgi?ledger=$ledger&ledgerCode=$ledgerCode&journal_command=create&journal_code=all_journals\">".$phrases->map("New_Transaction")."</a>");
  $doc->print("<a href=\"?app=Accounting&state=JournalEntry&command=display&ledger=$ledger&ledgerCode=$ledgerCode\">_</a>");
  $doc->print("\n | <a href=\"$jsUrl/journals.cgi?app=Accounting&state=Main&command=display&ledger=$ledger&ledgerCode=$ledgerCode\">Journals</a>");
  $doc->print("\n | <a href=\"$jsUrl/register.cgi?app=Accounting&state=Main&edit=false&command=&open=Open&ledger_open=true&ledger=$ledger&ledgerCode=$ledgerCode\">".$phrases->map("Registers")."</a>");
  $doc->print("\n | <a href=\"$jsUrl/journal_search.cgi?app=Accounting&state=Main&edit=false&command=&open=Open&ledger_open=true&ledger=$ledger&ledgerCode=$ledgerCode\">".$phrases->map("Search")."</a>");
  $doc->print("\n | <a href=\"$url?app=Accounting&state=InvJournal&command=display&ledger=$ledger&ledgerCode=$ledgerCode\">".$phrases->map("Investments")."</a>");
  $doc->print("\n <a href=\"$url?app=Accounting&state=".$doc->encodeString(string => "Investments::Journal")."&command=display&ledger=$ledger&ledgerCode=$ledgerCode\">new</a>");
  $doc->print("\n | <a href=\"$url?app=Accounting&state=Portfolio&command=display&ledger=$ledger&ledgerCode=$ledgerCode\">".$phrases->map("Portfolio")."</a>");
  $doc->print("\n | <a href=\"$url?app=Accounting&state=CGL&command=display&ledger=$ledger&ledgerCode=$ledgerCode\">".$phrases->map("Capital Gains")."</a>");
  $doc->print("\n | <a href=\"$url?app=Accounting&state=". $doc->encodeString(string => "Investments::WatchList")."&command=display&ledger=$ledger&ledgerCode=$ledgerCode\">".$phrases->map("Watchlists")."</a>");
  $doc->print("\n | <a href=\"$jsUrl/report.cgi?app=Accounting&state=Reports&command=display_for_running&ledger=$ledger&ledgerCode=$ledgerCode\">".$phrases->map("Reports")."</a>");
  $doc->print("\n | <a href=\"\" onclick=\"javascript: { windowName = getTimeStamp(); window.open('$url?app=Accounting&state=Config&windowName='+windowName+'&command=display&ledgerCode=$ledgerCode&ledgerOpen=1', windowName , 'width=$mainWidth,height=$mainHeight,scrollbars,status,toolbar'); return false; }\">".$phrases->map("Configuration")."</a>");
  $doc->print(<<"END_OF_CODE");
 |
    </td>
    <td align="right">
END_OF_CODE
  $doc->print("<font size=\"+2\"> <a href=\"$url?app=Accounting&state=Ledger&command=ledgerFrame&ledger=$ledger&ledgerCode=$ledgerCode&windowName=$windowName\" target=\"_top\" title=\"".$phrases->map("New_Frame")."\">F</a>");
  $doc->print(" | <a href=\"\" onclick=\"javascript: { windowName = getTimeStamp(); window.open('$url?app=Accounting&state=Ledger&windowName='+windowName+'&command=ledgerFrame&ledgerCode=$ledgerCode&ledger=$ledger', windowName , 'width=$mainWidth,height=$mainHeight,scrollbars,status,toolbar'); return false; }\" title=\"".$phrases->map("New_Window")."\">W</a>");
  $doc->print(" | <a href=\"$url?app=Accounting&state=Ledger&command=closeLedger&ledger=$ledger&ledgerCode=$ledgerCode&windowName=$windowName\" alt=\"".$phrases->map("Close_Window")."\" title=\"".$phrases->map("Close_Window")."\" target=\"_top\">X</a></font>");
  $doc->print(<<"END_OF_CODE");
 |
    </td>
</table>
END_OF_CODE

  return $doc;
}

sub ledgerFrame
{
  my $self = shift;
  my %args = ( @_ );
  my $errStr = "Portal::Accounting::Ledger->ledgerFrame()  - Error!<br>\n";
  my $siteDir = $self->{portalConfigObj}->{siteDir};
  my $phrases = $self->{langObj};
  my $ledger = $self->{input}->{ledger};
  my $ledgerCode = $self->{input}->{ledgerCode};
  #create frameset in object
  my $framedoc = HTMLObject::FrameSet->new();
  my $windowName;

  my $url = $self->{methods}->createBaseURL(type => "App", linkType => "cgi", appConfigObj => $self->{sessionObj}->{store}->{companyAppObj});

  if (defined $self->{input}->{windowName})
  {
    $windowName = $self->{input}->{windowName};
    #register window with portal!!!!!!
  }
  else
  {
    $self->setError( $self->errorMessage() . "Did not receive a windowName for frameset<br>\n");
    return undef;
  }
  $framedoc->setTitle("$ledger");

  my $inner_content;
  $inner_content = $framedoc->createNoframe($phrases->map("frameBrowser"));
  $inner_content .= $framedoc->createFrame(
    src => "$url?app=Accounting&state=Ledger&command=ledgerMenu&windowName=$windowName&ledger=$ledger&ledgerCode=$ledgerCode",
    frameborder => 'yes', scrolling => 'auto',
    name => $windowName . "Menu",
    bordercolor => 'white',
    marginwidth => '0',
    marginheight => '0',
    border => '1');
  $inner_content .= $framedoc->createFrame(
    src => "$url?app=Accounting&state=Ledger&command=display&ledger=$ledger&ledgerCode=$ledgerCode\"",
    frameborder => 'yes',
    scrolling => 'auto',
    name => $windowName . "Data",
    bordercolor => 'white',
    marginwidth => '1');
  $framedoc->outerFrameset(
    rows => '35, *',
    name => $windowName,
    content => $inner_content,
    border => '3',
    frameborder => 'yes');

  if (exists $self->{input}->{windowName})
  {# add this ledger to the window list.
    my $name = $self->{input}->{windowName};
    if ($name eq "")
    {
      $self->setError(errorString => $errStr . "Window name was not set!<br>\n");
      return undef;
    }
    $self->{sessionObj}->{store}->{windows}->{$name} = $name;
    $self->{sessionObj}->{store}->{changed} = 1;
  }

  return $framedoc;
}

sub editLedger
{
  my $self = shift;
  my %args = ( message => "", @_ );
  my $errStr = "Portal::Accounting::Ledger->editLedger()  - Error!<br>\n";
  my $siteDir = $self->{configObj}->{siteDir};
  my $phrases = $self->{langObj};
  my $windowName = $self->{input}->{windowName};

  my $dbLedger = Portal::Accounting::DB::Ledger->new(dbHandle => $self->{dbHandle}, langObj => $phrases );
  if ($dbLedger->didErrorOccur())
  {
    $self->setError($self->errorMessage() . $errStr . "Could not jump through hoops to get DB:Ledger!<br>\n". $dbLedger->errorMessage);
    return undef;
  }
  my %businessTypeS = $dbLedger->getBusinessTypeNames;
  my $dateSizes = 6;
  my $errorString = "";
  my $ledgerCode = "";
  my $ledgerObj;
  my $message = $args{message};
  my $nextCommand = "";
  my $ledgerOpen = (exists $self->{input}->{ledgerOpen} ? $self->{input}->{ledgerOpen} : 0);

  my $url = $self->{methods}->createBaseURL(type => "App", linkType => "cgi", appConfigObj => $self->{sessionObj}->{store}->{companyAppObj});
  my $portaljsUrl = $self->{methods}->createBaseURL(type => "Portal", linkType => "js");

  my $doc = HTMLObject::Normal->new();
  $doc->setHTMLInfo(dtd => 'strict');
  if (defined $self->{input}->{windowName})
  {
    #register window with portal!!!!!!  this will happen each time the screen refreshes for wrong user data
    if (!exists $self->{sessionObj}->{store}->{windows}->{$windowName})
    {
      $self->{sessionObj}->{store}->{windows}->{$windowName} = $windowName;
      $self->{sessionObj}->{store}->{changed} = 1;
    }
  }
  if ($self->{input}->{command} eq "createLedger")
  {
    $nextCommand = "saveCreateLedger";
    #initalize all params
    $self->{input}->{ledgerName} = "";
    $self->{input}->{ledgerCode} = "";
    $self->{input}->{ledgerColor} = "";
    $self->{input}->{businessType} = "";
    $self->{input}->{basis} = "";
    $self->{input}->{year} = "";
    $self->{input}->{month} = "01";
    $self->{input}->{day} = "01";
    $self->{input}->{fy} = "";
    $self->{input}->{start2} = "02-01";
    $self->{input}->{start3} = "03-01";
    $self->{input}->{start4} = "04-01";
    $self->{input}->{start5} = "05-01";
    $self->{input}->{start6} = "06-01";
    $self->{input}->{start7} = "07-01";
    $self->{input}->{start8} = "08-01";
    $self->{input}->{start9} = "09-01";
    $self->{input}->{start10} = "10-01";
    $self->{input}->{start11} = "11-01";
    $self->{input}->{start12} = "12-01";
  }
  elsif ($self->{input}->{command} eq "editLedger")
  {
    $ledgerCode = $self->{input}->{ledgerCode};
    if (not defined $ledgerCode)
    {
      $self->setError($self->errorMessage . "ledger not defined!<br>");
      return undef;
    }
    $nextCommand = "saveEditLedger";
    #get params from db object
    $ledgerObj = ($dbLedger->getLedgers(ledgerCode => $ledgerCode))[0];
    if ($dbLedger->didErrorOccur)
    {
      $self->setError("Error while retrieving ledger with code '$ledgerCode'" . $dbLedger->errorMessage);
      return undef;
    }
    if (!defined $ledgerObj)
    {
      $self->setError("ledgerObj not defined!" . $dbLedger->errorMessage);
      return undef;
    }
    if ($ledgerObj->didErrorOccur)
    {
      $self->setError("Error in ledger with code '$ledgerCode'.\n<br>" . $ledgerObj->errorMessage);
      return undef;
    }
    $self->{input}->{ledgerName} = $ledgerObj->{name};
    $self->{input}->{ledgerCode} = $ledgerCode;
    $self->{input}->{ledgerColor} = $ledgerObj->{color};
    $self->{input}->{businessType} = $ledgerObj->{businessType};
    $self->{input}->{cashOrAccrual} = $ledgerObj->{cashOrAccrual};
    $self->{input}->{basis} = $ledgerObj->{basis};
    $self->{input}->{fy} = (sort keys %{$ledgerObj->{knownYears}})[0] ;
    if ($ledgerObj->{knownYears}->{$self->{input}->{fy}} !~ m/^(\d{4})(\d{2})(\d{2})$/)
    {
      $message .= "Invalid knownYear: ".$ledgerObj->{knownYears}->{$self->{input}->{fy}};
    }
    $self->{input}->{year} = $ledgerObj->formatDate($ledgerObj->firstDate());
    $self->{input}->{fyMonth} = $2;
    $self->{input}->{fyDay} = $3;
    my @starts = $ledgerObj->getMonthStarts();
    if ($ledgerObj->didErrorOccur)
    {
      $message .= $ledgerObj->errorMessage;
    }
    if (not defined @starts || @starts != 12)
    {
      $message .= $phrases->map("Invlid_data_from")." ledgerObj->getMonthStarts <br>\n";
    }
    else
    {
      for (my $month = 2; $month <= 12; $month++)
      {
        $self->{input}->{"start$month"} = $starts[$month - 1];
        #$message .= " $month: ".$starts[$month -1 ]."  $self->{input}->{\"start$month\"} = $starts[$month - 1]<br>\n";
      }
    }
  }
  else
  {
    #this screen has been called with user data.  some of which is probably invalid
    #commnand is saveCreateLedger or saveEditLedger.  do not reset
    #$message has already been set.
    $nextCommand = $self->{input}->{command};
  }

  $doc->setFocus("body");
  $doc->print($message);
  $doc->print(<<"END_OF_CODE");
<form name="mainForm" action="$url">
<input type="hidden" name="app" value="Accounting">
<input type="hidden" name="state" value="Ledger">
<input type="hidden" name="command" value="$nextCommand">
<input type="hidden" name="ledgerOpen" value="$ledgerOpen">
<input type="hidden" name="windowName" value="$windowName">
<table width="100%" cellspacing="5" cellpadding="0" border="0">
  <tr>
    <td align="left" valign="top">
END_OF_CODE
  if ($self->{input}->{command} eq "createLedger" || $self->{input}->{command} eq "saveCreateLedger" )
  {
    $doc->setTitle($phrases->map("Create")." ".$phrases->map("Ledger"));
    $doc->print("      <b>".$phrases->map("LedgerName")."</b>&nbsp;<input type=\"text\" name=\"ledgerName\" size=\"15\" value=\"$self->{input}->{ledgerName}\"> &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;");
    $doc->print("      <b>".$phrases->map("LedgerCode")."</b>&nbsp;<input type=\"text\" name=\"ledgerCode\" size=\"5\" value=\"$self->{input}->{ledgerCode}\">");
  }
  else
  {
    $doc->setTitle($phrases->map("Edit")." ".$phrases->map("Ledger"));
    $doc->print("      <input type=\"hidden\" name=\"ledgerName\" value=\"$ledgerObj->{name}\">");
    $doc->print("      <input type=\"hidden\" name=\"ledgerCode\" value=\"$ledgerCode\">");
    $doc->print("      <b>".$phrases->map("LedgerName").":</b>&nbsp;$self->{input}->{ledgerName} &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;");
    $doc->print("      <b>".$phrases->map("LedgerCode").":</b>&nbsp;$self->{input}->{ledgerCode}");
  }
  my $ledgerColorPicker = $self->{methods}->generatePickerCode(type => "color", form => "mainForm",
                          itemName => "ledgerColor", itemValue => "$self->{input}->{ledgerColor}",
                          phrase => $phrases->map("LedgerColor"));
  if ($self->{methods}->didErrorOccur)
  {
    $self->setError($self->{methods}->errorMessage);
    return undef;
  }

  $self->{methods}->includeCoreJSFiles(doc => $doc);
  if ($self->{methods}->didErrorOccur)
  {
    $self->setError($self->{methods}->errorMessage);
    return undef;
  }

  $doc->print(<<"END_OF_CODE");
    </td>
  </tr>
  <tr>
    <td valign="top" align="left">
      $ledgerColorPicker
    </td>
  </tr>
  <tr>
    <td>
      <table width="100%"><tr>
      <td align="left" width="35%">
END_OF_CODE
  $doc->print("        <b>" . $phrases->map("BusinessType") . "</b><br>");
  $doc->print("        <select name=\"businessType\">\n");
  foreach my $type (sort keys %businessTypeS)
  {
    $doc->print("          <option value=\"$type\"");
    if ($self->{input}->{businessType} eq $type)
    {
      $doc->print(" selected");
    }
    $doc->print(">".$businessTypeS{$type}."\n");
  }
  $doc->print(<<"END_OF_CODE");
        </select>
      </td>
      <td align="left" width="65%">
        <table width="50%" border="1"><tr><td>
END_OF_CODE
  my $cashCheck = "";
  my $accCheck = "";
  if ($self->{input}->{cashOrAccrual} =~ /^cash$/i)
  {  $cashCheck = " checked";  }
  elsif ($self->{input}->{cashOrAccrual} =~ /^accrual$/i)
  {  $accCheck = " checked";  }
  else
  {
    $doc->print(" <b>Warning:  cashOrAccrual = '$self->{input}->{cashOrAccrual}'!</b><br>\n");
    $cashCheck = " checked";
  }
  $doc->print("        <input type=\"radio\" name=\"basis\" $cashCheck value=\"cash\"> ". $phrases->map("Cash")."<br>\n");
  $doc->print("        <input type=\"radio\" name=\"basis\" $accCheck value=\"accrual\"> ". $phrases->map("Accrual")."\n");
  $doc->print(<<"END_OF_CODE");
        </td></tr></table>
      </td>
      </tr></table>
    </td>
  </tr>
</table>
<table border="1" width="100%">
  <tr>
    <td align="left">
END_OF_CODE
  #if editing these fields cannot change.  use text instead of edit fields
  #this is messy but the quaqrters table is already messy.
  if ($self->{input}->{command} eq "createLedger" || $self->{input}->{command} eq "saveCreateLedger" )
  {
    $doc->print("      <b>" . $phrases->map("FiscalYear") . "</b> <input type=\"text\" name=\"fy\" size=\"6\" value=\"$self->{input}->{fy}\">");
    $doc->print("      <b>" . $phrases->map("Begins_on") . "</b><br>");
    $doc->print("      <b>" . $phrases->map("Year") . "</b> <input type=\"text\" name=\"year\" size=\"6\" value=\"$self->{input}->{year}\">");
    $doc->print("      <b>" . $phrases->map("Month") . "</b> <input type=\"text\" name=\"month\" size=\"3\" value=\"$self->{input}->{month}\">");
    $doc->print("      <b>" . $phrases->map("Day") . "</b> <input type=\"text\" name=\"day\" size=\"3\" value=\"$self->{input}->{day}\">");

    $doc->setFocus("javascript");

    # create the updateStartInfo() js method that will allow the user to specify the
    # starting month-day value and then propogate the changes to the 2-12 start fields.
    $doc->print(<<"END_OF_CODE");
function updateStartInfo()
{
  var d = document.mainForm;

  // get the first month and day values
  var month = d.month.value;
  var day = d.day.value;
  var n=new Number(month);

  // now update the start2-12 values to be month+1-day
  // take care to handle the month+1 > 12 case
  for (var i=2; i<13; i++)
  {
    // alert("i='"+i+"', month='"+n+"', day='"+day+"'");
    if (n + 1 > 12)
    {
      n = 1;
    }
    else
    {
      n++;
    }
    try
    {
      eval("d.start"+i+".value='"+(n.toString().length == 1 ? "0" : "") + n.toString() +"-"+day+"';");
    }
    catch(exception)
    {
      alert(exception);
    }
  }
}
END_OF_CODE

    $doc->setFocus("body");

    my $updateStartInfoPhrase = $phrases->map("UpdateStartInfo");
    $doc->print(<<"END_OF_CODE");
      <input type="button" value="$updateStartInfoPhrase" name="updateSI" onClick="updateStartInfo();">
    </td>
    <td>
      <table border="0" width="100%">
        <tr>
          <th align="left">
END_OF_CODE
    $doc->print("      " . $phrases->map("Month"));
    $doc->print(<<"END_OF_CODE");
          </th>
          <th align="center">
END_OF_CODE
    $doc->print("      " . $phrases->map("startDate"));
    $doc->print(<<"END_OF_CODE");
          </th>
        </tr>
END_OF_CODE
    for (my $index = 2; $index < 13; $index++)
    {
      my $startIndex = "start".$index;
      $doc->print(<<"END_OF_CODE");
        <tr>
          <td align="left">$index</td>
          <td align="center">
            <input type="text" name="$startIndex" value="$self->{input}->{$startIndex}" size="$dateSizes">
          </td>
        </tr>
END_OF_CODE
    }
    $doc->print(<<"END_OF_CODE");
      </table>
    </td>
  </tr>
</table>
END_OF_CODE
  }
  else
  {
    $doc->print("      <b>" . $phrases->map("FiscalYear") . ":</b> ".$self->{input}->{fy});
    $doc->print("      <b>" . $phrases->map("Begins_on") . ":</b> ".$self->{input}->{year});
    $doc->print(<<"END_OF_CODE");
    </td>
    <td>
      <table border="0" width="100%">
        <tr>
          <th align="left">
END_OF_CODE
    $doc->print("      " . $phrases->map("Month"));
    $doc->print(<<"END_OF_CODE");
    </th>
    <th align="center">
END_OF_CODE
    $doc->print("      " . $phrases->map("startDate"));
    $doc->print(<<"END_OF_CODE");
          </th>
        </tr>
END_OF_CODE
    for (my $index = 2; $index <= 12; $index++)
    {
      my $var = $self->{input}->{"start$index"};
      $doc->print(<<"END_OF_CODE");
        <tr>
          <td align="left">$index</td>
          <td align="center">
            $var
          </td>
        </tr>
END_OF_CODE
    }
    $doc->print(<<"END_OF_CODE");
      </table>
    </td>
  </tr>
</table>
END_OF_CODE
  }
  $doc->print("<input type=\"submit\" name=\"Submit\" value=\"".$phrases->map("Submit")."\">");
  $doc->print(<<"END_OF_CODE");
</form>
END_OF_CODE
  return $doc;
}

sub saveLedger
{  #check for saveCreateLedger to insert, otherwise update
  my $self = shift;
  my %args = ( @_ );
  my $windowName = $self->{input}->{windowName};

  my $url = $self->{methods}->createBaseURL(type => "App", linkType => "cgi", appConfigObj => $self->{sessionObj}->{store}->{companyAppObj});
  my $command = $self->{input}->{command};
  my $errStr = "Portal::Accounting::Ledger->saveLedger()  - Error!<br>\n";
  my $ledgerObj;
  my $dbLedger = Portal::Accounting::DB::Ledger->new(dbHandle => $self->{dbHandle}, langObj => $self->{langObj});
  if ($dbLedger->didErrorOccur())
  {
    $self->setError($self->errorMessage() . $errStr . "Could not initialize module!<br>\n");
    return $self->editLedger(message => $errStr . "Could not initialize module Portal::Accounting::DB::Ledger->new!<br>\n");
  }
  my $knownYears = {};
  my $ledgerOpen = (exists $self->{input}->{ledgerOpen} ? $self->{input}->{ledgerOpen} : 0);
  #if error along the way submit to editLedger with message =>
  my $status;
  my @start = ();
  if ($self->{input}->{command} eq "saveCreateLedger")
  {
    foreach my $var ("fy", "year", "month", "day")
    {
      if ((not exists $self->{input}->{$var}) || ($self->{input}->{$var} eq "") || ($self->{input}->{$var} !~ /^\d+$/))
      {
         return $self->editLedger(message => "$var is invalid. Please try again!");
      }
    }
    #convert first day of this year into last day of this year for knownYears
    my @tempdate = Date::Calc::Add_Delta_Days($self->{input}->{year} + 1, $self->{input}->{month}, $self->{input}->{day}, -1);
    $knownYears->{$self->{input}->{fy}} = "$tempdate[0]-$tempdate[1]-$tempdate[2]";
    $start[0] = $self->{input}->{year} - $self->{input}->{fy};
    $start[1] = $self->{input}->{month}."-".$self->{input}->{day};
    $start[2] = $self->{input}->{start2};
    $start[3] = $self->{input}->{start3};
    $start[4] = $self->{input}->{start4};
    $start[5] = $self->{input}->{start5};
    $start[6] = $self->{input}->{start6};
    $start[7] = $self->{input}->{start7};
    $start[8] = $self->{input}->{start8};
    $start[9] = $self->{input}->{start9};
    $start[10] = $self->{input}->{start10};
    $start[11] = $self->{input}->{start11};
    $start[12] = $self->{input}->{start12};

    #put data gathered into ledger Object
    $ledgerObj = Portal::Accounting::Objects::Ledger->new(
      langObj => $self->{langObj}, name => $self->{input}->{ledgerName},
      code => $self->{input}->{ledgerCode}, color => $self->{input}->{ledgerColor},
      cashOrAccrual => $self->{input}->{basis}, businessType => $self->{input}->{businessType},
      knownYears => $knownYears, start => \@start
      );
    if ($ledgerObj->didErrorOccur)
    {
      return $self->editLedger(message => $ledgerObj->errorMessage . "\n<br>Something is invalid!");  #!!!! phrase-map
    }
#    my $endDate = $self->{input}->{year} . "-" . $ledgerObj->{q4End};

    #check to see if ledger already exists
    my @ledgerByName = $dbLedger->getLedgers(ledgerName => $ledgerObj->{name});
    if ($dbLedger->didErrorOccur)
    {
      return $self->editLedger(message => "Name caused error while looking for '$ledgerObj->{name}'.  Cannot create Ledger!<br>\n" . $dbLedger->errorMessage);
    }
    my @ledgerByCode = $dbLedger->getLedgers(ledgerCode => $ledgerObj->{code});
    if ($dbLedger->didErrorOccur)
    {
      return $self->editLedger(message => "Code caused error while looking for '$ledgerObj->{code}'.  Cannot create Ledger!<br>\n" . $dbLedger->errorMessage);
    }
    if (scalar @ledgerByCode > 0)
    {
      return $self->editLedger(message => "Code '$ledgerObj->{code}' already exists. Cannot create Ledger!<br>\n");
    }
    elsif (scalar @ledgerByName > 0)
    {
      return $self->editLedger(message => "Name '$ledgerObj->{name}' already exists. Cannot create Ledger!<br>\n");
    }
    else
    {
      $status = $dbLedger->createLedger(langObj => $self->{langObj}, ledger => $ledgerObj) ;
    }
  }
  elsif ($self->{input}->{command} eq "saveEditLedger")
  {
    $status = $dbLedger->updateLedger(ledgerCode => $self->{input}->{ledgerCode}, color => $self->{input}->{ledgerColor}, businessType => $self->{input}->{businessType}, cashOrAccrual => $self->{input}->{basis});
  }
  else
  {
    return $self->editLedger(message => "You can't get here because the command is $self->{input}->{command}");
  }
  if (!$status)
  {
    return $self->editLedger(message => "Error!" . $dbLedger->errorMessage. "ledgerCode: $self->{input}->{ledgerCode}");
  }
  #if ok, output page with form to submit to state=Ledger,command=ledgerFrame #!!!!

  my $phrases = $self->{langObj};
  my $commandAction = $phrases->map('updated');
  if ($self->{input}->{command} eq 'saveCreateLedger')
  {
    $commandAction = $phrases->map('created');
  }
  my $doc = HTMLObject::Normal->new();
  if ((not defined $doc) || $doc->didErrorOccur)
  {
    $self->setError($self->errorMessage . "Error. Could not created HTMLObject<br>\n");
    return undef;
  }

  my $openLedgerURL = "$url?app=Accounting&state=Ledger&command=ledgerFrame&ledger=".$self->{input}->{ledgerName}."&ledgerCode=".$self->{input}->{ledgerCode}."&windowName=' + window.name";

  $doc->setTitle($phrases->map("Update")."/".$phrases->map("Create")." " . $phrases->map('ledger'));
  $doc->setFocus("body");
  $doc->print("<h1>".$phrases->map('Ledger')." <font color=\"$self->{input}->{ledgerColor}\">$self->{input}->{ledgerName}</font> ". $commandAction."</h1><br>\n");
  if (!$ledgerOpen)
  {
    $doc->print("<p><a href=\"\" onclick=\"window.location = '$openLedgerURL; return false;\">".$phrases->map("Open")."</a> ");
    $doc->print($phrases->map('Ledger') . " " . $phrases->map("to_begin_working").".</p>");

    $doc->setFocus("javascript");
    $doc->print(<<"END_OF_CODE");
function onloadWork()
{
  if (window.opener.refreshWindow)
  {
    window.opener.refreshWindow();
  }
  window.setTimeout("window.location='$openLedgerURL", 2000); // open ledger in 2 seconds.
}
END_OF_CODE
  }
  else
  {
    # remove this ledger from the window list.
    if (!exists $self->{sessionObj}->{store}->{windows}->{$windowName})
    {
      $self->setError(errorString => $self->errorMessage . $errStr . "Window = '$windowName' does not exist!<br>\n");
      return undef;
    }
    delete $self->{sessionObj}->{store}->{windows}->{$windowName};
    $self->{sessionObj}->{store}->{changed} = 1;

    $doc->print("<p><a href=\"\" onclick=\"window.close(); return false;\">".$phrases->map("Close_Window")."</a> ");
    $doc->setFocus("javascript");
    $doc->print(<<"END_OF_CODE");
function onloadWork()
{
  if (window.opener.refreshWindow)
  {
    window.opener.refreshWindow();
  }
  window.setTimeout("window.close();", 3000); // close in 3 seconds.
}
END_OF_CODE
  }
  $doc->setOnload(code => "onloadWork();");

  return $doc;
}

sub accountSummary
{
  my $self = shift;

  my $doc = HTMLObject::Normal->new();
  my $mainDB = Portal::Accounting::DB::Main->new(dbHandle => $self->{dbHandle}, langObj => $self->{langObj});
  if ($mainDB->didErrorOccur)
  {
    $self->prefixError("Could not initialize DB::Main object!<br>".$mainDB->errorMessage."\n");
    return undef;
  }

  $doc->setTitle($self->{langObj}->map("Account")." ".$self->{langObj}->map("Summary"));
  $doc->setFocus("head");
  $doc->print(<<"ENDOF");
      <style type="text/css">
      <!--
        .CashSummaryTitle { font-weight: bold; text-decoration: none; }
        .CashSummaryTable { }
        .CashSummaryName { text-transform: capitalize; }
        .CashSummaryBalance {text-align: right; }
        .CashSummaryReconcile { }
        .CashSummaryLastEntry { }
        .negative { color: #ff0000; }
      -->
      </style>
ENDOF
  $doc->setFocus("body");
  $doc->print(<<"ENDOF");
  <center>
  <table width="100%" border="0">
        <tr>
            <td width="50%">
              <center>
ENDOF
  my $html = $mainDB->getCashAccountSummary(ledgerCode => $self->{input}->{ledgerCode});
  if ($mainDB->didErrorOccur)
  {
    $html = $mainDB->errorMessage;
  }
  if (not defined $html)
  {
    $html = "Error during Cash account Summary!<br>" . $mainDB->errorMessage();
  }
  $doc->print($html);
  $doc->print(<<"ENDOF");
              </center>
            </td>
          </tr>
        </table>
</center>
ENDOF

  return $doc;
}

sub deleteLedger
{
  my $self = shift;

  my $ledgerCode = $self->{input}->{ledgerCode};
  if (not defined $ledgerCode)
  {
    $self->setError($self->errorMessage . "ledger not defined!<br>");
    return undef;
  }

  my $doc = HTMLObject::Normal->new();
  my $ledgerDB = Portal::Accounting::DB::Ledger->new(dbHandle => $self->{dbHandle}, langObj => $self->{langObj});
  if ($ledgerDB->didErrorOccur)
  {
    $self->prefixError("Could not initialize DB::Ledger object!<br>".$ledgerDB->errorMessage."\n");
    return undef;
  }

  $doc->setTitle($self->{langObj}->map("Delete")." ".$self->{langObj}->map("Ledger"));

  my $result = $ledgerDB->deleteLedger(ledgerCode => $ledgerCode);
  if (!$result)
  {
    $doc->print("Error!  Failed to Delete Ledger with code = '$ledgerCode'!<br><br>Error = '" . $ledgerDB->errorMessage . "'.");
    return $doc;
  }
  else
  {
    $doc->setFocus("body");
    $doc->print(<<"END_OF_CODE");
Ledger with code = '$ledgerCode' deleted!<br><br>
Refreshing xiwa main menu...
END_OF_CODE
    $doc->setFocus("javascript");
    $doc->print(<<"END_OF_CODE");
function onloadWork()
{
  window.opener.refreshWindow();
  window.setTimeout("window.close();", 2000); // close in 2 seconds.
}
END_OF_CODE
    $doc->setOnload(code => "onloadWork();");
  }

  return $doc;
}

1;
__END__

=head2

  NOTE:  All data fields are accessible by specifying the object
         and pointing to the data member to be modified on the
         left-hand side of the assignment.
         Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

JT Moree
PC & Web Xperience, Inc. (mailto:admin@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal(3), Portal::Accounting(3)

=cut
