# Menu.pm - The Object Class that provides a Account Object
# Created by JT Moree, 2/07/2002
#$Id: Menu.pm,v 1.10 2004/01/28 22:52:14 james Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2002 http://www.pcxperience.org  All rights reserved.

package Portal::Accounting::Objects::Menu;
use strict;
use Portal::Accounting::Data::Config;
use Portal::Base;
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Portal::Base Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '0.02';

=head1 NAME

Menu - Object used to build a Account Menu Class.

=head1 SYNOPSIS

  use Portal::Accounting::Objects::Menu;
  my $menuObj = Portal::Accounting::Objects::Menu->new(methods,
                sessionObj, langObj, userObj);
  if ($menuObj->didErrorOccur())
  {
    die $menuObj->errorMessage();
  }

=head1 DESCRIPTION

Menu is a class to display the menu for the accounting system.
The menu will eventually only show the modules that the user
has priveleges to access.

=head1 Exported FUNCTIONS

B<NOTE>: I<bool> = 1(true), 0(false)

=over 4

=item scalar new(methods, sessionObj, langObj, userObj, input, portalSession)

 Creates a new instance of the Portal::Accounting::Objects::Menu module.
 See Portal::Base(3) for a listing of required arguments.

 requires: methods - methods object,
           sessionObj - sessionObj for app,
           langObj - langObj for app,
           userObj - userObj for user,
           input - the input hash
           portalSession - the Portal Session

=cut

sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  my %args = ( methods => undef, sessionObj => undef, langObj => undef, userObj => undef, input => undef, portalSession => undef, @_ );

  if ($self->didErrorOccur)
  {
    $self->prefixError();
    return $self;
  }

  # instantiate anything unique to this module
  $self->{methods} = $args{methods};
  $self->{sessionObj} = $args{sessionObj};
  $self->{langObj} = $args{langObj};
  $self->{userObj} = $args{userObj};
  $self->{input} = $args{input};
  $self->{portalSession} = $args{portalSession};

  # do validation
  if (!$self->isValid())
  {
    # the error is set in the isValid() method.
    return $self;
  }

  # do anything else you might need to do.

  return $self;
}

=item bool isValid(void)

 Returns 0 or 1 to indicate if the object is valid.
 The error will be available via errorMessage().

=cut

sub isValid
{
  my $self = shift;

  # make sure our Parent class is valid.
  if (!$self->SUPER::isValid())
  {
    $self->prefixError();
    return 0;
  }

  # validate our parameters.
  if (! defined $self->{methods})
  {
    $self->missing("methods");
  }
  if (! defined $self->{sessionObj})
  {
    $self->missing("sessionObj");
  }
  if (! defined $self->{langObj})
  {
    $self->missing("langObj");
  }
  if (! defined $self->{userObj})
  {
    $self->missing("userObj");
  }
  if (! defined $self->{input})
  {
    $self->missing("input");
  }
  if (! defined $self->{portalSession})
  {
    $self->missing("portalSession");
  }

  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->postfixError($self->genErrorString("all"));
    return 0;
  }

  return 1;
}

=item string print(location, state, command, doc)

  requires: location - (menubar, sidemenu), state, command, doc
  returns:  generated Menubar string

  generates the specified Menu.

=cut

sub print
{
  my $self = shift;
  my %args = ( location => "menubar", state => "Main", command => "display", doc => undef, @_ );
  my $location = $args{location};
  my $state = $args{state};
  my $command = $args{command};
  my $doc = $args{doc};
  my $result = "";
  my $errStr = "Portal::Accounting::Objects::Menu->print() - Error!<br>\n";

  if ($location !~ /^(menubar|sidemenu)$/)
  {
    $self->setError(errorString => $errStr . "location = '$location' is invalid!<br>\n");
    return $result;
  }
  if ($state !~ /^(Main|DisplayItems|Ledger|InvJournal|Portfolio|QIFImport|DBUpgrade|StockPrices|CGL|Config::Monitor|Config)$/)
  {
    $self->setError(errorString => $errStr . "state = '$state' is invalid!<br>\n");
    return $result;
  }
  if ($command !~ /^(.+)$/)
  {
    $self->setError(errorString => $errStr . "command = '$command' is invalid!<br>\n");
    return $result;
  }
  if (! defined $doc)
  {
    $self->setError(errorString => $errStr . "doc needs be to defined!<br>\n");
    return $result;
  }

  my $url = $self->{methods}->createBaseURL(type => "App", linkType => "cgi", appConfigObj => $self->{sessionObj}->{store}->{companyAppObj});
  if ($self->{methods}->didErrorOccur)
  {
    $self->setError(errorString => $errStr . $self->{methods}->errorMessage);
    return $result;
  }

  my $appPart = "?app=Accounting";

  my %commandUrls = (
             Globals => {
               close => $appPart . "&state=Main&command=close",
               refresh => $appPart . "&state=$state&command=$command",
               help => "?app=Portal&state=Help&command=display&helpApp=Accounting&content=Normal,$state",
             },
             Main => {
             },
             DisplayItems => {
             },
             Ledger => {
             },
             InvJournal => {
             },
             Portfolio => {
             },
             QIFImport => {
             },
             DBUpgrade => {
             },
             StockPrices => {
             },
             CGL => {
             },
             "Config::Monitor" => {
             },
             Config => {
               close => $appPart . "&state=Config&command=close&windowName=$self->{input}->{windowName}",
               refresh => $appPart . "&state=$state&command=$command&ledgerCode=" . (exists $self->{input}->{ledgerCode} ? $doc->encodeString(string => $self->{input}->{ledgerCode}) : "") . "&windowName=$self->{input}->{windowName}&ledgerOpen=" . ($self->{input}->{ledgerOpen} ? $self->{input}->{ledgerOpen} : 0),
             },
  );

  my %commandPhrases = (
             Globals => {
               close => "Close",
               refresh => "Refresh",
               help => "Help",
             },
             Main => {
             },
             DisplayItems => {
             },
             Ledger => {
             },
             InvJournal => {
             },
             Portfolio => {
             },
             QIFImport => {
             },
             DBUpgrade => {
             },
             StockPrices => {
             },
             CGL => {
             },
             "Config::Monitor" => {
             },
             Config => {
               close => "Close",
             },
  );

  my %titlePhrases = (
             Main => {
               display => "Main_Menu",
               close => "Main_Menu",
             },
             DisplayItems => {
             },
             Ledger => {
             },
             InvJournal => {
             },
             Portfolio => {
             },
             QIFImport => {
             },
             DBUpgrade => {
             },
             StockPrices => {
             },
             CGL => {
             },
             "Config::Monitor" => {
             },
             Config => {
               display => "Config_Menu",
             },
  );

  # now gather the command and title phrases and store them as {command}_phrase.
  foreach my $stateKey (keys %commandPhrases)
  {
    foreach my $commandKey (keys %{$commandPhrases{$stateKey}})
    {
      my $index = $commandKey . "_phrase";
      $commandPhrases{$stateKey}{$index} = $self->langObj->map($commandPhrases{$stateKey}{$commandKey});
    }
  }
  foreach my $stateKey (keys %titlePhrases)
  {
    foreach my $commandKey (keys %{$titlePhrases{$stateKey}})
    {
      my $index = $commandKey . "_phrase";
      $titlePhrases{$stateKey}{$index} = $self->langObj->map($titlePhrases{$stateKey}{$commandKey});
    }
  }

  my $titleCommand = $titlePhrases{$state}{$command."_phrase"};
  my $title = sprintf($self->langObj->map("title"), $Portal::Accounting::VERSION, $titleCommand);

  $doc->setTitle($title);

  # build up the menuItems arrays
  my @itemsLeft = ();
  my @itemsRight = ();
  my @itemsCenter = ();

  if ($location eq "menubar")
  {
    if ($state =~ /^(Main|Config)$/)
    {
      my $closeUrl = $url . (exists $commandUrls{$state}{close} ? $commandUrls{$state}{close} : $commandUrls{Globals}{close});
      my $closePhrase = (exists $commandPhrases{$state}{"close_phrase"} ? $commandPhrases{$state}{"close_phrase"} : $commandPhrases{Globals}{"close_phrase"});
      my $helpPhrase = $commandPhrases{Globals}{"help_phrase"};
      my $refreshUrl = $url . (exists $commandUrls{$state}{refresh} ? $commandUrls{$state}{refresh} : $commandUrls{Globals}{refresh});
      my $refreshPhrase = $commandPhrases{Globals}{"refresh_phrase"};

      $itemsLeft[0] = Portal::Objects::MenuItem->new(type => "link", url => "$closeUrl", langObj => $self->{langObj},
                      text => $closePhrase, title => $closePhrase, onMouseOver => "window.status='$closePhrase'; return true;", onClick => (exists $commandUrls{$state}{close} ? "return true;" : "return validateClose();"));
      $itemsLeft[1] = Portal::Objects::MenuItem->new(type => "seperator", langObj => $self->{langObj});
      $itemsLeft[2] = Portal::Objects::MenuItem->new(type => "link", url => "$url$commandUrls{Globals}{help}", langObj => $self->{langObj},
                      text => $helpPhrase, onMouseOver => "window.status='$helpPhrase'; return true;",
                      onClick => "showHelp('help_Accounting" . "_$self->{portalSession}->{store}->{windowTitleTimestamp}', '$url$commandUrls{Globals}{help}', 480, 640); return false;", title => $helpPhrase);
      $itemsLeft[3] = Portal::Objects::MenuItem->new(type => "seperator", langObj => $self->{langObj});
      $itemsLeft[4] = Portal::Objects::MenuItem->new(type => "link", url => "$refreshUrl", langObj => $self->{langObj},
                      text => $refreshPhrase, onMouseOver => "window.status='$refreshPhrase'; return true;", title => "$refreshPhrase");
    }
    else
    {
    }

    # now validate they created ok.
    for (my $i=0; $i < scalar @itemsLeft; $i++)
    {
      if ($itemsLeft[$i]->didErrorOccur)
      {
        $self->setError(errorString => $errStr . "itemsLeft[$i]\n<br>" . $itemsLeft[$i]->errorMessage);
        return $result;
      }
    }
    for (my $i=0; $i < scalar @itemsRight; $i++)
    {
      if ($itemsRight[$i]->didErrorOccur)
      {
        $self->setError(errorString => $errStr . "itemsRight[$i]\n<br>" . $itemsRight[$i]->errorMessage);
        return $result;
      }
    }
    $result = $self->{methods}->displayMenu(doc => $doc, orientation => "horizontal",
                    itemsLeft => \@itemsLeft, itemsRight => (scalar @itemsRight ? \@itemsRight : undef), indent => 2);
  }
  elsif ($location eq "sidemenu")
  {
    #$itemsLeft[0] = Portal::Objects::MenuItem->new(type => "link", url => "$changePasswordUrl", langObj => $self->{langObj},
    #                text => $changePasswordPhrase, title => $changePasswordPhrase, onMouseOver => "window.status='$changePasswordPhrase'; return true;");
    #$itemsLeft[1] = Portal::Objects::MenuItem->new(type => "text", langObj => $self->{langObj},
    #                text => "$editPhrase $userPropertiesPhrase");
    #$itemsLeft[2] = Portal::Objects::MenuItem->new(type => "link", url => "$userPreferencesUrl", langObj => $self->{langObj},
    #                text => "$editPhrase $userPreferencesPhrase", onMouseOver => "window.status='$editPhrase $userPreferencesPhrase'; return true;", title => "$editPhrase $userPreferencesPhrase");
    if ($state ne "Main")
    {
    #  $itemsLeft[3] = Portal::Objects::MenuItem->new(type => "seperator", langObj => $self->{langObj});
    #  $itemsLeft[4] = Portal::Objects::MenuItem->new(type => "link", url => "$mainMenuUrl", langObj => $self->{langObj},
    #                  text => $mainMenu, onMouseOver => "window.status='$mainMenu'; return true;", title => "$mainMenu");
    }

    # now validate they created ok.
    for (my $i=0; $i < scalar @itemsLeft; $i++)
    {
      if ($itemsLeft[$i]->didErrorOccur)
      {
        $self->setError(errorString => $errStr . "itemsLeft[$i]\n<br>" . $itemsLeft[$i]->errorMessage);
        return $result;
      }
    }
    $result = $self->{methods}->displayMenu(orientation => "vertical",
                    itemsLeft => \@itemsLeft, indent => 6);
  }
  if ($self->{methods}->didErrorOccur)
  {
    $self->setError(errorString => $errStr . $self->{methods}->errorMessage);
    return $result;
  }

  return $result;
}

=back

=cut

1;
__END__

=head1 NOTE

 All data fields are accessible by specifying the object
 and pointing to the data member to be modified on the
 left-hand side of the assignment.
 Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

Xperience, Inc. (mailto:admin@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal(3), Portal::Accounting(3), Portal::Base(3)

=cut
