# PortfolioAccountObject.pm - The Object Class that provides a PortfolioAccount Object
# Created by James Pattie & Jason A. Pattie, 11/07/2000.
#$Id: PortfolioAccountObject.pm,v 1.1.1.1 2001/08/09 19:04:47 moreejt Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.

package Portal::Accounting::Objects::PortfolioAccountObject;
use strict;
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Exporter AutoLoader);
@EXPORT = qw();
	
$VERSION = '0.01';

# new 
# instantiates an instance of a PortfolioAccountObject
# takes: accountName, accountNum, totalCost, totalMarketValue, totalGainLoss, status, stocks
sub new
{
  my $that = shift;
  my $class = ref($that) || $that;
  my $self = bless {}, $class;
  my %args = ( accountName => "", accountNum => 0, totalCost => "0.00", totalMarketValue => "0.00", totalGainLoss => "0.00", status => "c", stocks => undef, @_ );
  
  $self->{accountName} = $args{accountName};
  $self->{accountNum} = $args{accountNum};
  $self->{totalCost} = $args{totalCost};
  $self->{totalMarketValue} = $args{totalMarketValue};
  $self->{totalGainLoss} = $args{totalGainLoss};
  $self->{status} = $args{status};
  $self->{stocks} = $args{stocks};

  # do validation
  $self->{error} = !$self->isValid;  
  if ($self->{error})
  {
    $self->{errorString} = "PortfolioAccountObject: new() - Error!<br>\n" . $self->{errorString};
  }

  return $self;
}

# isValid - Returns 0 or 1 to indicate if the object is valid.
sub isValid
{
  my $self = shift;
  my $error = 0;
  my $errorString = "";

  # do validation code here.
  if ($self->{accountNum} < 0)
  {
    $error = 1;
    $errorString .= "accountNum = '$self->{accountNum}' is invalid!<br>\n";
  }
  if ($self->{accountName} < 0)
  {
    $error = 1;
    $errorString .= "accountName = '$self->{accountName}' is invalid!<br>\n";
  }
  if ($self->{status} !~ /^(o|c)$/)
  {
    $error = 1;
    $errorString .= "status = '$self->{status}' is invalid!<br>\n";
  }
  #if (scalar @{$self->{stocks}} == 0)
  #{
  #  $error = 1;
  #  $errorString .= "stocks must be defined!<br>\n";
  #}
  if ($self->{totalCost} !~ /^-?\d+\.\d+$/)
  {
    $error = 1;
    $errorString .= "totalCost = '$self->{totalCost}' is invalid!<br>\n";
  }
  if ($self->{totalMarketValue} !~ /^-?\d+\.\d+$/)
  {
    $error = 1;
    $errorString .= "totalMarketValue = '$self->{totalMarketValue}' is invalid!<br>\n";
  }
  if ($self->{totalGainLoss} !~ /^-?\d+\.\d+$/)
  {
    $error = 1;
    $errorString .= "totalGainLoss = '$self->{totalGainLoss}' is invalid!<br>\n";
  }

  $self->{error} = $error;
  $self->{errorString} = "PortfolioAccountObject: isValid() - Error!<br>\n" if $error;
  $self->{errorString} .= $errorString;

  return !$error;
}

sub DESTROY
{
  my $self = shift;
}

sub AUTOLOAD
{
  my $self = shift;
  my $type = ref($self) || die "$self is not an object";
  my $name = $AUTOLOAD;
  $name =~ s/.*://;	# strip fully-qualified portion
  unless (exists $self->{$name})
  {
    die "Can't access `$name' field in object of class $type";
  }
  if (@_)
  {
    return $self->{$name} = shift;
  }
  else
  {
    return $self->{$name};
  }
}

# setError
# requires: errorString
# returns: nothing
sub setError
{
  my $self = shift;
  my %args = ( errorString => "", @_ );
  my $errorString = $args{errorString};
  
  $self->{errorString} = $errorString;
  $self->{error} = 1;
}

# didErrorOccur
# returns the value of error.
sub didErrorOccur
{
  my $self = shift;

  return $self->{error};
}

# errorMessage
# returns the value of errorString
sub errorMessage
{
  my $self = shift;

  return $self->{errorString};
}

1;
__END__

=head1 NAME

PortfolioAccountObject - Object used to build a PortfolioAccount Object Class.

=head1 SYNOPSIS

  use Portal::Accounting::Objects::PortfolioAccountObject;
  my @stocks = array of PortfolioStockObject(s).
  my $portfolioAcctObj = 
  Portal::Accounting::Objects::PortfolioAccountObject->new(
          accountName => "NYSE", accountNum => 1234, 
          totalCost => "100.00", totalMarketValue => "110.00",
          totalGainLoss => "10.00", status => "c", stocks => \@stocks);
  if ($portfolioAcctObj->didErrorOccur())
  {
    die $portfolioAcctObj->errorMessage();
  }

=head1 DESCRIPTION

PortfolioAccountObject is a PortfolioAccount class.  It represents the
Account and all stocks and their lots in the Portfolio View.  It is the
high-level overview of what this account is doing (investment wise).

=head1 Exported FUNCTIONS

  scalar new(accountName, accountNum, totalCost, totalMarketValue,
             totalGainLoss, status, stocks)
    Creates a new instance of the
    Portal::Accounting::Objects::PortfolioAccountObject object.
    status = (o-open, c-closed)
    stocks = reference to array of PortfolioStockObjects.

  bool isValid(void)
    Returns 0 or 1 to indicate if the object is valid.  The error will
    be available via errorMessage().

  void setError(errorString)
    Sets error = 1 and $self->{errorString} = errorString.

  scalar didErrorOccur(void)
    Returns the value of error.

  scalar errorMessage(void)
    Returns the value of errorString.


  NOTE:  All data fields are accessible by specifying the object
         and pointing to the data member to be modified on the
         left-hand side of the assignment.
         Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

PC & Web Xperience, Inc. (mailto:admin@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal(3), Portal::Accounting(3)

=cut
